Package: ContourFunctions
Type: Package
Title: Create Contour Plots from Data or a Function
Version: 0.1.2
Authors@R: person("Collin", "Erickson", 
  email = "collinberickson@gmail.com",
  role = c("aut", "cre"))
Description: Provides functions for making contour plots.
  The contour plot can be created from grid data, a function,
  or a data set. If non-grid data is given, then a Gaussian
  process is fit to the data and used to create the contour plot.
License: GPL-3
RoxygenNote: 7.3.1
Imports: rlang
Depends: ggplot2, lhs, rmarkdown
Suggests: covr, knitr, laGP, locfit, mgcv, mlegp, testthat
VignetteBuilder: knitr
URL: https://github.com/CollinErickson/ContourFunctions
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2024-09-09 23:52:39 UTC; colli
Author: Collin Erickson [aut, cre]
Maintainer: Collin Erickson <collinberickson@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-10 15:00:12 UTC
Built: R 4.6.0; ; 2026-04-28 14:05:13 UTC; unix

Package: ContourFunctions
Type: Package
Title: Create Contour Plots from Data or a Function
Version: 0.1.2
Authors@R: person("Collin", "Erickson", 
  email = "collinberickson@gmail.com",
  role = c("aut", "cre"))
Description: Provides functions for making contour plots.
  The contour plot can be created from grid data, a function,
  or a data set. If non-grid data is given, then a Gaussian
  process is fit to the data and used to create the contour plot.
License: GPL-3
RoxygenNote: 7.3.1
Imports: rlang
Depends: ggplot2, lhs, rmarkdown
Suggests: covr, knitr, laGP, locfit, mgcv, mlegp, testthat
VignetteBuilder: knitr
URL: https://github.com/CollinErickson/ContourFunctions
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2024-09-09 23:52:39 UTC; colli
Author: Collin Erickson [aut, cre]
Maintainer: Collin Erickson <collinberickson@gmail.com>
Repository: CRAN
Date/Publication: 2024-09-10 15:00:12 UTC
Built: R 4.6.0; ; 2026-04-26 11:28:06 UTC; unix

# Generated by roxygen2: do not edit by hand

export(cf)
export(cf_4dim)
export(cf_data)
export(cf_func)
export(cf_grid)
export(cf_highdim)
export(cm.colors.strong)
export(csa)
export(eval_over_grid_with_batch)
export(gcf)
export(gcf_data)
export(gcf_func)
export(gcf_grid)
export(multicolor.title)
export(text_plot)
importFrom(ggplot2,ggplot)
importFrom(grDevices,cm.colors)
importFrom(graphics,.filled.contour)
importFrom(graphics,Axis)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,close.screen)
importFrom(graphics,contour)
importFrom(graphics,layout)
importFrom(graphics,lcm)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,screen)
importFrom(graphics,split.screen)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(lhs,randomLHS)
importFrom(rlang,.data)
importFrom(rmarkdown,html_vignette)
importFrom(stats,lm)
importFrom(stats,predict)
importFrom(utils,capture.output)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ContourFunctions

<!-- badges: start -->

[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/ContourFunctions)](https://cran.r-project.org/package=ContourFunctions)
[![Codecov test
coverage](https://codecov.io/gh/CollinErickson/ContourFunctions/branch/master/graph/badge.svg)](https://app.codecov.io/gh/CollinErickson/ContourFunctions?branch=master)
[![R-CMD-check](https://github.com/CollinErickson/ContourFunctions/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/CollinErickson/ContourFunctions/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

This is an R package that provides simple functions for creating contour
plots.

## Overview

The main functions are:

- `cf_grid`: Makes a contour plot from grid data.

- `cf_func`: Makes a contour plot for a function.

- `cf_data`: Makes a contour plot for a data set by fitting a Gaussian
  process model.

- `cf`: Passes arguments to `cf_function` or `cf_data` depending on
  whether the first argument is a function or numeric.

All of these functions make the plot using base graphics by default. To
make plots using ggplot2, add the argument `gg=TRUE`, or put g in front
of the function name. E.g., `gcf_data(...)` is the same as
`cf_data(..., gg=TRUE)`, and makes a similar plot to `cf_data` but using
ggplot2.

There are two functions for making plots in higher dimensions:

- `cf_4dim`: Plots functions with four inputs by making a series of
  contour plots.

- `cf_highdim`: Plots for higher dimensional inputs by making a contour
  plot for each pair of input dimensions and holding the other inputs
  constant or averaging over them.

## Installation

    # It can be installed like any other package
    install.packages("ContourFunctions")

    # Or the the development version from GitHub:
    # install.packages("devtools")
    devtools::install_github("CollinErickson/contour")

## Usage

Plot a grid of data:

``` r
library(ContourFunctions)
a <- b <- seq(-4*pi, 4*pi, len = 27)
r <- sqrt(outer(a^2, b^2, "+"))
cf_grid(a, b, cos(r^2)*exp(-r/(2*pi)))
```

![](tools/README-cf_grid-1.png)<!-- -->

Plot a function with two input dimensions:

``` r
f1 <- function(r) cos(r[1]^2 + r[2]^2)*exp(-sqrt(r[1]^2 + r[2]^2)/(2*pi))
cf_func(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi))
```

![](tools/README-cf_func-1.png)<!-- -->

Using data with two inputs and an output, fit a Gaussian process model
and show the contour surface with dots where the points are:

``` r
set.seed(0)
x <- runif(20)
y <- runif(20)
z <- exp(-(x-.5)^2-5*(y-.5)^2)
cf_data(x,y,z)
#> Fitting with laGP since n <= 200
```

![](tools/README-cf_data-1.png)<!-- -->

For more than two input dimensions:

``` r
friedman <- function(x) {
  10*sin(pi*x[1]*x[2]) + 20*(x[3]-.5)^2 + 10*x[4] + 5*x[5]
}
cf_highdim(friedman, 5, color.palette=topo.colors)
```

![](tools/README-cf_highdim-1.png)<!-- -->

For (three or) four inputs dimensions:

``` r
cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])})
```

![](tools/README-cf_4dim-1.png)<!-- -->

---
output: github_document
---

<!-- README.md is generated from README.Rmd. Please edit that file -->

```{r, echo = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "tools/README-"
)
```


# ContourFunctions

[![Travis-CI Build Status](https://travis-ci.org/CollinErickson/ContourFunctions.svg?branch=master)](https://travis-ci.org/CollinErickson/ContourFunctions)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/ContourFunctions)](https://cran.r-project.org/package=ContourFunctions)
[![Codecov test coverage](https://codecov.io/gh/CollinErickson/ContourFunctions/branch/master/graph/badge.svg)](https://codecov.io/gh/CollinErickson/ContourFunctions?branch=master)

This is an R package that provides simple functions for creating contour plots.

## Overview

The main functions are:

* `cf_grid`: Makes a contour plot from grid data.

* `cf_func`: Makes a contour plot for a function.

* `cf_data`: Makes a contour plot for a data set by fitting a Gaussian process model.

* `cf`: Passes arguments to `cf_function` or `cf_data` depending on whether the first argument is a function or numeric.

All of these functions make the plot using base graphics by default.
To make plots using ggplot2, add the argument `gg=TRUE`,
or put g in front of the function name.
E.g., `gcf_data(...)` is the same as `cf_data(..., gg=TRUE)`,
and makes a similar plot to `cf_data` but using ggplot2.

There are two functions for making plots in higher dimensions:

* `cf_4dim`: Plots functions with four inputs by making a series
of contour plots.

* `cf_highdim`: Plots for higher dimensional inputs by making a
contour plot for each pair of input dimensions and holding the other
inputs constant or averaging over them.


## Installation

```
# It can be installed like any other package
install.packages("ContourFunctions")

# Or the the development version from GitHub:
# install.packages("devtools")
devtools::install_github("CollinErickson/contour")
```

## Usage

Plot a grid of data:

```{r cf_grid}
library(ContourFunctions)
a <- b <- seq(-4*pi, 4*pi, len = 27)
r <- sqrt(outer(a^2, b^2, "+"))
cf_grid(a, b, cos(r^2)*exp(-r/(2*pi)))
```

Plot a function with two input dimensions:

```{r cf_func}
f1 <- function(r) cos(r[1]^2 + r[2]^2)*exp(-sqrt(r[1]^2 + r[2]^2)/(2*pi))
cf_func(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi))
```

Using data with two inputs and an output,
fit a Gaussian process model and show the
contour surface with dots where the points are:

```{r cf_data}
set.seed(0)
x <- runif(20)
y <- runif(20)
z <- exp(-(x-.5)^2-5*(y-.5)^2)
cf_data(x,y,z)
```

For more than two input dimensions:

```{r cf_highdim}
friedman <- function(x) {
  10*sin(pi*x[1]*x[2]) + 20*(x[3]-.5)^2 + 10*x[4] + 5*x[5]
}
cf_highdim(friedman, 5, color.palette=topo.colors)
```

For (three or) four inputs dimensions:

```{r cf_4dim}
cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])})
```

#' Create contour plot from grid data using ggplot2
#' 
#' The same as cf_grid_screen but uses ggplot2 for the plot.
#'
#' @param x  x values, must form grid with y.
#' If not given, it is assumed to be from 0 to 1.
#' @param y  y values, must form grid with x.
#' If not given, it is assumed to be from 0 to 1.
#' @param z  z values at grid locations
#' @param xlim  x limits for the plot.
#' @param ylim  y limits for the plot.
#' @param zlim  z limits for the plot.
#' @param with_lines Should lines be added on top of 
#' contour to show contours?
#' @param lines_only Should no fill be used, only contour lines?
#' @param bins Number of lines used when using `with_lines` or `lines_only`
#' @param interpolate Will smooth out contours
#' @param levels  a set of levels which are used to partition the range of z.
#' Must be strictly increasing (and finite). Areas with z values between
#' consecutive levels are painted with the same color.
#' @param nlevels  if levels is not specified, the range of z, values is
#' divided into approximately this many levels.
#' @param color.palette  a color palette function to be used to assign colors
#' in the plot. Defaults to cm.colors. Other options include rainbow,
#' heat.colors, terrain.colors, topo.colors, and function(x) {gray((1:x)/x)}.
#' @param col  an explicit set of colors to be used in the plot. This argument
#' overrides any palette function specification. There should be one less
#' color than levels
#' @param asp  the y/x aspect ratio, see plot.window.
#' @param las  the style of labeling to be used. The default is to use
#' horizontal labeling.
#' @param bar Should a bar showing the output range and colors be shown on the right?
#' @param pts Points to plot on top of contour
#' @param reset.par Should the graphical parameters be reset before exiting?
#' Usually should be unless you need to add something to the plot
#' afterwards and bar is TRUE.
#' @param pretitle Text to be preappended to end of plot title
#' @param posttitle Text to be appended to end of plot title
#' @param main Title for the plot
#' @param mainminmax  whether the min and max values should be shown
#' in the title of plot
#' @param mainminmax_minmax Whether [min,max]= should be shown in title
#' or just the numbers
#' @param afterplotfunc Function to call after plotting, such as adding
#' points or lines.
#' @param cex.main The size of the main title. 1.2 is default.
#' @param ...  additional graphical parameters, currently only passed to title().
#' 
#' @return ggplot2 object
#' @export
#'
#' @examples
#' x <- y <- seq(-4*pi, 4*pi, len = 27)
#' r <- sqrt(outer(x^2, y^2, "+"))
#' gcf_grid(cos(r^2)*exp(-r/(2*pi)))
#' gcf_grid(r, color.palette=heat.colors, bar=TRUE)
#' gcf_grid(r, color.palette=function(x) {gray((1:x)/x)}, bar=TRUE)
gcf_grid <-  function (x = seq(0, 1, length.out = nrow(z)), 
                       y = seq(0, 1,length.out = ncol(z)), z,
                       xlim = range(x, finite = TRUE),
                       ylim = range(y, finite = TRUE),
                       zlim = range(z, finite = TRUE),
                       with_lines=FALSE,
                       lines_only=FALSE,
                       bins=8, # number of contour lines
                       interpolate=TRUE,
                       levels = pretty(zlim, nlevels), nlevels = 20,
                       color.palette = cm.colors.strong,
                       col = color.palette(length(levels) - 1),
                       asp = NA, las = 1,
                       bar=F, pts=NULL, reset.par=TRUE,
                       pretitle="", posttitle="", main=NULL,
                       mainminmax=!bar, mainminmax_minmax=TRUE,
                       afterplotfunc=NULL,
                       cex.main=par()$cex.main,
                       ...) {
  if (missing(z)) {
    if (!missing(x)) {
      if (is.list(x)) {
        z <- x$z
        y <- x$y
        x <- x$x
      } else {
        z <- x
        x <- seq.int(0, 1, length.out = nrow(z))
      }
    } else {
      stop("no 'z' matrix specified")
    }
  } else if (is.list(x)) {
    y <- x$y
    x <- x$x
  }
  if (any(diff(x) <= 0) || any(diff(y) <= 0)) {
    stop("increasing 'x' and 'y' values expected")
  }
  
  if (any(diff(x) <= 0) || any(diff(y) <= 0)) {
    stop("increasing 'x' and 'y' values expected")
  }
  
  
  t2 <- cbind(expand.grid(x, y), tz=c(z))
  p <- ggplot2::ggplot() +
    # scale_fill_gradientn(colours=c("cyan","white","magenta"))
    ggplot2::scale_fill_gradientn(colours=col)
  if (!lines_only) {
    p <- p + ggplot2::geom_raster(ggplot2::aes_string(x="Var1", y="Var2", fill = "tz"),
                         t2, interpolate=interpolate)
  }
  
  # Add contour lines
  if (with_lines) {
    p <- p + ggplot2::geom_contour(ggplot2::aes_string(x="Var1", y="Var2", z = "tz"),
                                   bins=bins,
                                   color="black",
                                   t2)
  }
  if (lines_only) { # Only color lines if only lines
    p <- p + ggplot2::geom_contour(ggplot2::aes_string(x="Var1", y="Var2", z = "tz",
                                                       color="..level.."),
                                   bins=bins,
                                   t2)
  }
  
  if (bar) {
    # Remove title on legend
    p <- p + ggplot2::theme(legend.title = ggplot2::element_blank())
  }
  if (!bar) {
    p <- p + ggplot2::theme(legend.position = "none") 
  }
  
  # Remove axes labels
  p <- p + ggplot2::theme(axis.title.x = ggplot2::element_blank()) + 
    ggplot2::theme(axis.title.y = ggplot2::element_blank())
  
  # Cut off extra space
  # p <- p + ggplot2::coord_cartesian(xlim=c(min(x), max(x)), ylim=c(min(y), max(y)), expand=F)
  p <- p + ggplot2::coord_cartesian(xlim=xlim, ylim=ylim, expand=F)
  
  
  if (mainminmax | !is.null(main)) {
    # Unable to put colors in title
    # p <- p + gg_make_multicolor_title()
    gtitle <- pretitle
    if (mainminmax_minmax) {
      gtitle <- paste0(gtitle, "[min,max]=")
    }
    gtitle <- paste0(gtitle, "[",signif(min(z),3),",", signif(max(z),3),"]", posttitle)
    p <- p + ggplot2::ggtitle(gtitle) + 
      ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0.5))
    # Max/min annotations since colors don't work
    minind <- arrayInd(which.min(z), dim(z))
    xminz <- x[minind[1]]
    yminz <- y[minind[2]]
    xminz <- min(max(xminz, xlim[1]+.025*(xlim[2]-xlim[1])), xlim[2]-.025*(xlim[2]-xlim[1]))
    yminz <- min(max(yminz, ylim[1]+.015*(ylim[2]-ylim[1])), ylim[2]-.015*(ylim[2]-ylim[1]))
    p <- p + ggplot2::annotate("text", x = xminz, y = yminz, label = signif(min(z),3))
    maxind <- arrayInd(which.max(z), dim(z))
    xmaxz <- x[maxind[1]]
    ymaxz <- y[maxind[2]]
    xmaxz <- min(max(xmaxz, xlim[1]+.025*(xlim[2]-xlim[1])), xlim[2]-.025*(xlim[2]-xlim[1]))
    ymaxz <- min(max(ymaxz, ylim[1]+.015*(ylim[2]-ylim[1])), ylim[2]-.015*(ylim[2]-ylim[1]))
    p <- p + ggplot2::annotate("text", x = xmaxz, y = ymaxz, label = signif(max(z),3))
  }
  
  if (!is.null(pts)) {
    if (!is.matrix(pts)) { # if not a matrix, make it a matrix by row
      if (is.numeric(pts) && (length(pts)%%2==0)) {
        pts <- matrix(pts, ncol=2, byrow = T)
      }
    }
    # points(pts, pch=19)
    pts <- as.data.frame(pts)
    colnames(pts) <- c("V1", "V2")
    p <- p + ggplot2::geom_point(ggplot2::aes_string("V1","V2"), data=pts)
  }
  
  if (!is.null(afterplotfunc)) {
    afterplotfunc()
  }
  
  if (reset.par) {# Either reset parameters
  } else { # or return it so user can do it later
    stop("Can't reset later, this returns the plot")
    # return(reset.par.func)
  }
  p
}

context("test-gcf_grid")

test_that("gcf_grid works", {
  x <- y <- seq(-4*pi, 4*pi, len = 27)
  r <- sqrt(outer(x^2, y^2, "+"))
  expect_error(p <- gcf_grid(cos(r^2)*exp(-r/(2*pi))), NA)
  expect_is(p, "gg")
  expect_is(p, "ggplot")
  
  # x,y,z in one list
  expect_error(gcf_grid(list(x=x,y=y,z=cos(r^2)*exp(-r/(2*pi)))), NA)
  # x,y in one list, z separate
  expect_error(gcf_grid(list(x=x,y=y),z=cos(r^2)*exp(-r/(2*pi))), NA)
  # bad x/y values
  expect_error(gcf_grid(rev(x), y, cos(r^2)*exp(-r/(2*pi))))
  expect_error(gcf_grid(x, rev(y), cos(r^2)*exp(-r/(2*pi))))
  # Bar
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), bar=T), NA)
  # pts
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), pts=matrix(c(.1,.2,.3,.4),2,2)), NA)
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), pts=c(.1,.2,.3,.4)), NA)
  # Can't reset.par
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), reset.par = F))
  # afterplotfunc, not useful, maybe should remove
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), afterplotfunc = function(){("after")}), NA)
  
  # Lines
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), lines_only = T), NA)
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), with_lines = T), NA)
  expect_error(gcf_grid(cos(r^2)*exp(-r/(2*pi)), with_lines = T, lines_only = T), NA)
  
  # Errors
  expect_error(gcf_grid()) # No z given
  expect_error(gcf_grid(x)) # No z given
  expect_error(gcf_grid(rev(x), y, cos(r^2)*exp(-r/(2*pi))))
  expect_error(gcf_grid(x, rev(y), cos(r^2)*exp(-r/(2*pi))))
  
})

test_that("gcf_func", {
  p1 <- gcf_func(function(xx){exp(-sum((xx-.5)^2/.1))})
  p2 <- cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, gg=T)
  expect_is(p1, "gg")
  expect_is(p2, "gg")
})

test_that("gcf_data", {
  x <- runif(20)
  y <- runif(20)
  z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
  p1 <- gcf_data(x,y,z)
  p2 <- cf_data(x,y,z, gg=T)
  expect_is(p1, "gg")
  expect_is(p2, "gg")
})

test_that("gcf", {
  # Func
  p1 <- gcf(function(xx){exp(-sum((xx-.5)^2/.1))})
  p2 <- cf(function(xx){exp(-sum((xx-.5)^2/.1))}, gg=T)
  expect_is(p1, "gg")
  expect_is(p2, "gg")
  
  # Data
  x <- runif(20)
  y <- runif(20)
  z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
  p3 <- gcf_data(x,y,z)
  p4 <- cf_data(x,y,z, gg=T)
  expect_is(p3, "gg")
  expect_is(p4, "gg")
})

#' Makes plot title using specified colors for the text
#' @param main  Text to put in main title of plot
#' @param col.main  Colors to use for the text
#' @param collapse  What to put between elements of main, defaults to "" but " " might be appropriate
#' @param cex.main The size of the main title. 1.2 is default.
#' @examples 
#' plot(1:4)
#' multicolor.title(c('Black, ','red, ','green'),c(1,2,3))
#' @export
multicolor.title <- function(main,col.main, collapse='', cex.main=par()$cex.main) {
  if (length(main) != length(col.main)) {stop('main and col must have same length')}
  n <- length(main)
  if(n==1) {
    title(bquote(.(main[1])),col.main=col.main[1], cex.main=cex.main)
  } else {
    # print first
    title(bquote(.(main[1]) * phantom(.(paste0(main[2:n],collapse=collapse)))),col.main=col.main[1], cex.main=cex.main)
    
    # print middle
    if(n > 2) {
      for(i in 2:(n-1)) {
        title(bquote(phantom(.(paste0(main[1:(i-1)],collapse=collapse))) * .(main[i]) * phantom(.(paste0(main[(i+1):n],collapse=collapse)))),col.main=col.main[i], cex.main=cex.main) 
      }
    }
    
    # print last
    title(bquote(phantom(.(paste0(main[1:(n-1)],collapse=collapse))) * .(main[n])),col.main=col.main[n], cex.main=cex.main)
  }
}

#' Make contour plot from data or function
#' 
#' Simpler function for making contours with cf package.
#' Won't give argument completion, so all must be specified
#'
#' @param ... Arguments to be passed to cf_func or cf_data based on 
#' data type of first argument. If D is given as argument, then it
#' is passed to cf_highdim.
#' @param gg Should ggplot2 be used instead of base graphics?
#'
#' @return Whatever is returned from other function, probably nothing.
#' Will be a ggplot2 object if using gg=TRUE.
#' @export
#'
#' @examples
#' cf(function(x){x[1]^2 - x[2]})
#' x <- runif(20)
#' y <- runif(20)
#' z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
#' cf(x,y,z)
#' cf(function(x){x[1]^2 - x[2]}, D=3)
cf <- function(..., gg=FALSE) {
  dots <- list(...)
  if (is.function(dots[[1]])) {
    if ("D" %in% names(dots)) {
      cf_highdim(...)
    } else {
      cf_func(..., gg=gg)
    }
  } else if (is.numeric(dots[[1]])) {
    cf_data(..., gg=gg)
  } else {
    stop("Data not recognized. Use cf_func for function or 
         cf_data for data or cf_grid for full grid of data.")
  }
}

#' Close all open screens
#' 
#' Closes the screens open, which happens
#' when plotting with `split.screen` is interrupted.
#' It often happens when there is a error while plotting.
#' When you try to plot
#' the next thing it gives an error.
#' Running this function will reset the plot screen.
#' It just does `close.screen(all.screens=TRUE)` but is faster to type.
#' 
#' @param silent Should the output of `close.screen` not be returned?
#' 
#' @examples
#' # Split screen into fourths
#' split.screen(c(2,2))
#' hist(rnorm(100))
#' screen(2)
#' hist(runif(100))
#' # Use csa() to go back to normal plotting
#' csa()
#' hist(rexp(100))
#' @export
csa <- function(silent=FALSE) {
  if (silent) { # Suppresses "FALSE" if already closed
    invisible(close.screen(all.screens=TRUE))
  } else {
    close.screen(all.screens=TRUE)
  }
}

context("non-ggplot tests")

test_that("cf_grid", {
  a <- b <- seq(-4*pi, 4*pi, len = 27)
  r <- sqrt(outer(a^2, b^2, "+"))
  cf_grid(cos(r^2)*exp(-r/(2*pi)))
  cf_grid(cos(r^2)*exp(-r/(2*pi)), bar=T)
  cf_grid(cos(r^2)*exp(-r/(2*pi)), bar=T, mainminmax = T)
  # Give in list x,y,z
  cf_grid(list(x=a,y=b,z=cos(r^2)*exp(-r/(2*pi))))
  # Give in nothing, get error
  expect_error(cf_grid())
  cf_grid(x=list(x=a,y=b), z=cos(r^2)*exp(-r/(2*pi)))
  expect_error(cf_grid(rev(x), y, cos(r^2)*exp(-r/(2*pi))))
  expect_error(cf_grid(x, rev(y), cos(r^2)*exp(-r/(2*pi))))
  expect_error(cf_grid(cos(r^2)*exp(-r/(2*pi)), xaxis=T, yaxis=F), NA)
  expect_error(cf_grid(cos(r^2)*exp(-r/(2*pi)), xaxis=F, yaxis=T), NA)
  expect_error(cf_grid(a,b,z=c(cos(r^2)*exp(-r/(2*pi)))))
  cf_grid(matrix(as.integer(3*cos(r^2)*exp(-r/(2*pi))),27,27))
  cf_grid(cos(r^2)*exp(-r/(2*pi)), pts=c(.1,.2,.3,.4,.5,.6))
  cf_grid(cos(r^2)*exp(-r/(2*pi)), afterplotfunc=function(){text(.4,.6,labels="Text added!")})
  expect_true(is.function(cf_grid(cos(r^2)*exp(-r/(2*pi)), reset.par = F)))
  
  # csa
  expect_error(csa(), NA)
  expect_error(csa(silent=T), NA)
})

test_that("cf_data", {
  x <- runif(20)
  y <- runif(20)
  z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
  cf_data(x,y,z)
  cf_data(x,y,z, bar=T)
  cf(x,y,z)
  cf_data(x,y,z, fit="mlegp")
  cf_data(x,y,z, xylim=c(-1,2))
  cf_data(cbind(x,y),y=z)
  cf_data(cbind(x,y),z=z)
  cf_data(cbind(x,y,z))
  
  # Errors
  expect_error(cf_data(cbind(x,y,y),y=z))
  expect_error(cf_data(cbind(x,y,y),z=z))
  expect_error(cf_data(cbind(x,y)))
  expect_error(cf_data(cbind(x,y,z,z)))
})

test_that("cf_func", {
  cf_func(function(xx){exp(-sum((xx-.5)^2/.1))})
  cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=T)
  cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, main="Title check")
  cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=T, main="Title check")
  cf_func(function(xx){exp(-sum((xx-.5)^2/.1))},pts=matrix(c(.2,.2,.2,.8,.8,.2,.8,.8),byrow=T,ncol=2))
  cf(function(xx){exp(-sum((xx-.5)^2/.1))})
  
  # xylim
  cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, xylim = c(-1,2))
  
  # use lines
  cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, use_lines = T)
  
  # out.col.name
  expect_error(cf_func(function(xx){data.frame(y1=exp(-sum((xx-.5)^2/.1)), y2=1)}))
  expect_error(cf_func(function(xx){data.frame(y1=exp(-sum((xx-.5)^2/.1)), y2=1)}, out.col.name = "y1"), NA)
  # out.name from list
  expect_error(cf_func(function(xx){list(y1=exp(-sum((xx-.5)^2/.1)), y2=1)}, out.name = "y1"), NA)
})

test_that("cf_4dim", {
  
  expect_error(cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])}), NA)
  
  # Add pts
  expect_error(cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])}, pts=matrix(c(.1,.2,.3,.4,.5,.6,.7,.8),2,4)), NA)
  expect_error(cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])}, pts=matrix(c(.1,.2,.3,.4,.5,.6,.7,.8,.9,1),2,5)))
  
  # Not same scale, this is a lot slower
  expect_error(cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])}, same_scale = F), NA)
  
  expect_error(
    cf_4dim(function(x) x[1]*x[3] + sin(x[2]*x[4]), color.palette=heat.colors,
         nover1=3, nover2=8, cex.var_names = .5), NA)
         
  expect_error(
    cf_4dim(function(x) x[1]*x[3] + sin(x[2]*x[4]), color.palette=topo.colors,
         nover1=3, nover2=8, cex.var_names = 1, over_srt = c(90,0),
         edge_width=c(.1, .2), nlevels = 5),
    NA)
})

test_that("cf highdim", {
  
  expect_error(cf_highdim(function(x) {x[1]^2 + exp(x[2])}, D=3), NA)
  
  friedman <- function(x) {
   10*sin(pi*x[1]*x[2]) + 20*(x[3]-.5)^2 + 10*x[4] + 5*x[5]
  }
  expect_error(cf_highdim(friedman, 5, color.palette=topo.colors), NA)
  expect_error(cf_highdim(friedman, 5, 
            color.palette=function(x) {gray((1:x)/x)},
            nlevels=10),
            NA)

  expect_error({
    # Average over background dimensions, use higher reps to reduce noise.
    f1 <- function(x) {x[1] + x[2]^2 + x[3]^3}
    cf_highdim(f1, 4, average=TRUE, average_reps=1e2, n=10)
    f1b <- function(x) {x[,1] + x[,2]^2 + x[,3]^3}
    cf_highdim(f1b, 4, average=TRUE, average_reps=1e2, n=10, batchmax=Inf)
    cf_highdim(f1b, 4, average_reps=1e2, n=10, batchmax=Inf,
              color.palette = topo.colors, nlevels=3)
    }, NA)
  
  # Error when pts has wrong num of columns
  expect_error(cf_highdim(function(x) {x[1]^2 + exp(x[2])}, D=3, pts=matrix(1:4,2,2)))
  
  # Batch max between 1 and nrows
  expect_error(cf_highdim(function(x) {if (is.matrix(x)){x[,1]^2+exp(x[,2])} else {x[1]^2 + exp(x[2])}}, D=3, batchmax=10), NA)
  # Batch max between 1 and nrows AND average
  expect_error(cf_highdim(function(x) {if (is.matrix(x)){x[,1]^2+exp(x[,2])} else {x[1]^2 + exp(x[2])}}, D=3, batchmax=10, average=T), NA)
  
  # Not same scale
  expect_error(cf_highdim(function(x) {x[1]^2 + exp(x[2])}, D=3, same_scale = F), NA)
  
  # From cf
  expect_error(cf(function(x) {x[1]^2 + exp(x[2])}, D=3), NA)
})

test_that("cf error", {
  expect_error(cf("you shouldn't give in text here!"))
})

#' Contour plot from data
#' 
#' Makes filled contour plot from data without sidebar by interpolating 
#' with a Gaussian process model.
#' This is the same as `cf_data` except it will use ggplot2 to
#' make the plot.
#' 
#' @param x  either just x data, x and y data, or x, y and z data
#' @param y  either y data, z data, or null
#' @param z  either z data or null
#' @param xlim  x limits for the contour plot, will be set to data limits +- 5\% if not specified
#' @param ylim  y limits for the contour plot, will be set to data limits +- 5\% if not specified
#' @param xylim x and y limits for the contour plot
#' @param fit Method to fit a model with. Current options are laGP (default)
#' and mlegp. laGP is faster but might cause trouble.
#' @param gg If FALSE, will use base graphics by calling cf_func()
#' @param ...  passed to cf_func
#' @importFrom utils capture.output
#' @examples 
#' x <- runif(20)
#' y <- runif(20)
#' z <- exp(-(x-.5)^2-5*(y-.5)^2)
#' gcf_data(x,y,z)
#' @export
gcf_data <- function(x, y=NULL, z=NULL,
                    xlim=NULL, ylim=NULL, xylim=NULL,
                    fit="",
                    gg=TRUE,
                    ...) {
  cf_data(x=x, y=y, z=z, xlim=xlim, ylim=ylim, xylim=xylim,
          fit=fit, gg=gg, ...)
}

#' Plot 2D contour slices of four dimensional functions
#' 
#' Plots a grid of contour plots.
#' Each contour plot is a contour over two dimensions with the remaining
#' two dimensions set to a value.
#' See cf_highdim for functions with more than 4 dimensions.
#'
#' @param func A four-dimensional function to plot contours of
#' @param over Indices of the dimensions used for the outer grid
#' @param nover Number of grid points for the outer grid dimensions
#' @param nover1 Number of grid points for the first outer grid dimension
#' @param nover2 Number of grid points for the second outer grid dimension
#' @param low Low input value for each dimension
#' @param high High input value for each dimension
#' @param n Number of points in grid on each dimension
#' @param same_scale Should all contour plots be on the same scale?
#' Takes longer since it has to precalculate range of outputs.
#' @param batchmax  number of datapoints that can be computed at a time
#' @param pts Matrix of points to show on plot
#' @param nlevels Number of levels in contour scale
#' @param color.palette Color palette used for contour plots
#' @param var_names Variable names to add to plot
#' @param bar Should a bar be added on right when all on same_scale?
#' @param key.axes key for bar plot
#' @param key.title  statements which add titles for the plot key.
#' @param axes axes
#' @param edge_width How wide should edges with variable names be?
#' As proportion of screen section to left of bar.
#' Either single value for both edges, or length two vector.
#' @param bar_width How wide should bar section of plot be?
#' @param cex.var_names Size of var_names printed on edges.
#' @param over_srt Degrees of rotation for the axis labels. Vector of length two.
#' @param ... Arguments passed to cf_func, and then probably through to cf_grid
#'
#' @importFrom graphics contour mtext
#' @return NULL
#' @export
#'
#' @examples
#' cf_4dim(
#'   function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])}
#' )
#' 
#' cf_4dim(function(x) x[1]*x[3] + sin(x[2]*x[4]), color.palette=heat.colors,
#'         nover1=3, nover2=8, cex.var_names = .5)
#'         
#' cf_4dim(function(x) x[1]*x[3] + sin(x[2]*x[4]), color.palette=topo.colors,
#'         nover1=3, nover2=8, cex.var_names = 1, over_srt = c(90,0),
#'         edge_width=c(.1, .2), nlevels = 5)
cf_4dim <- function(func,
                    over=c(1,2),
                    nover=5, nover1=nover, nover2=nover,
                    # over1=seq(0,1,length.out=nover1),
                    # over2=seq(0,1,length.out=nover2),
                    low=rep(0,4), high=rep(1,4),
                    same_scale=TRUE,
                    n=20,
                    batchmax=1,
                    var_names=c(expression(), 
                                lapply(1:4,
                                       function(ti) bquote(x[.(ti)]))),
                    pts=NULL,
                    axes=TRUE, key.axes, key.title,
                    nlevels=20,
                    color.palette=cm.colors.strong,
                    edge_width=.04, cex.var_names=1.3,
                    bar=TRUE,
                    bar_width=.2,
                    over_srt=c(0,90),
                    ...) {
  
  d1 <- (1:4)[-over][1] # d1 and d2 are the dimensions of the contour plots
  d2 <- (1:4)[-over][2] # over are the dimensions for the outer set of plots
  over1=seq(low[over[1]],high[over[1]],length.out=nover1)
  over2=seq(low[over[2]],high[over[2]],length.out=nover2)
  
  begin_screen <- screen()
  if (!is.null(pts)) {
    if (ncol(pts) != 4) {stop("pts must have 4 columns")}
  }
  # Make version of function for just two dimensions
  tfij <- function(x2, over1x, over2x) {
    mid2 <- rep(NaN, 4)
    mid2[-over] <- x2
    mid2[over[1]] <- over1[i]
    mid2[over[2]] <- over2[j]
    func(mid2)
  }
  # Get this function as a function
  get_tfij <- function(over1x,over2x) {
    function(x) {
      tfij(x2=x, over1x=over1x, over2x=over2x)
    }
  }
  if (same_scale) {
    # Put all plots on same scale, need to know max and min values before
    #  beginning plot, so it is twice as slow.
    zmin <- Inf
    zmax <- -Inf
    zlist <- list()
    for (j in 1:nover2) {
      zlist[[j]] <- list()
      for (i in 1:nover1) {
        zij <- eval_over_grid_with_batch(seq(low[d1],high[d1], l=n),
                                         seq(low[d2],high[d2], l=n),
                                         fn=get_tfij(over1x=over1[i],
                                                     over2x=over2[j]),
                                         batchmax)
        zlist[[j]][[i]] <- zij
        zmin <- min(zmin, min(zij))
        zmax <- max(zmax, max(zij))
      }
    }
    zlim <- c(zmin, zmax)
    levels <- pretty(zlim, nlevels)
    col <- color.palette(length(levels) - 1)
  }
  
  if (bar && same_scale) {
    # Make bar on right side
    bar_screens <- split.screen(matrix(c(0, 3/4, 0, 1, 3/4, 1, 0, 1), byrow=T, ncol=4))
    screen(bar_screens[2])
    # levels <- pretty(zlim, nlevels)
    # col <- color.palette(length(levels) - 1)
    okmar <- par()$mar
    kmar <- numeric(4) #mar.orig
    kmar[4L] <- 2.5#mar[2L] # right
    # kmar[1] <- 2.2 # bottom
    # kmar[3] <- .83 #if (mainminmax | !is.null(main)) 1.3 else .3 #1.3#1.3 # top
    # kmar[2L] <- 3#0#1 # left
    par(mar = kmar)
    kmai <- par("mai")
    # kdin <- par("din")
    leftmai <- .1
    kmai2 <- c(.1,
               leftmai,
               .1,
               par("mai")[4])
    par(mai = kmai2)
    plot.new()
    plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i", 
                yaxs = "i")
    rect(0, levels[-length(levels)], 1, levels[-1L], col = col)
    if (missing(key.axes)) {
      if (axes) 
        axis(4, las=1)
    }
    else key.axes
    box()
    if (!missing(key.title))
      key.title
    # mar <- mar.orig
    par(mar=okmar)
    close.screen(bar_screens[2])
    screen(bar_screens[1])
  }
  
  # Do this after bar so bar is separate
  if (length(edge_width) == 1) {edge_width <- c(edge_width, edge_width)}
  outer_screens <- split.screen(
    matrix(c(0,edge_width[2],edge_width[1],1,
             edge_width[2],1,edge_width[1],1,
             0,edge_width[2],0,edge_width[1],
             edge_width[2],1,0,edge_width[1]), byrow=T, ncol=4))
  screen(outer_screens[2])
  
  # Split screen for grid of plots
  par(mar=c(1,1,1,1))
  screen.numbers <- split.screen(c(nover2, nover1), erase = FALSE)
  current_screen_index <- 1
  current_screen <- screen.numbers[current_screen_index]
  
  for (j in nover2:1) {
    for (i in 1:nover1) {
      screen(current_screen)
      if (same_scale) {
        # Already calculated values, so pass them to cf_grid
        cf_grid(x=seq(low[d1], high[d1], length.out=n),
                y=seq(low[d2], high[d2], length.out=n),
                z=zlist[[j]][[i]],
                mainminmax=FALSE, xaxis=F&&(j==4), yaxis=F&&(i==1), #plot.axes=F,
                xlim=c(low[d1],high[d1]), ylim=c(low[d2],high[d2]),
                zlim=zlim, pts=pts[,c(d1, d2)],
                nlevels=nlevels, levels=levels,
                color.palette=color.palette, col=col,
                ...)
      } else {
        cf_func(get_tfij(over1x=over1[i], over2x=over2[i]), batchmax=batchmax,
                mainminmax=FALSE, xaxis=F, yaxis=F, #plot.axes=F,
                xlim=c(low[d1],high[d1]), ylim=c(low[d2],high[d2]),
                pts=pts[,c(d1, d2)],
                ...)
      }
      current_screen_index <- current_screen_index + 1
      current_screen <- screen.numbers[current_screen_index]
    }
  }
  close.screen(n = screen.numbers)
  screen(outer_screens[1])
  left_screens <- split.screen(c(nover2, 1))
  for (j in nover2:1) {
    screen(left_screens[nover2+1-j])
    text_plot(bquote(.(a)==.(b), where=list(a=var_names[[over[2]]], b=over2[j])), cex=cex.var_names, srt=over_srt[2])
  }
  close.screen(left_screens)
  screen(outer_screens[4])
  right_screens <- split.screen(c(1, nover1))
  for (i in 1:nover1) {
    screen(right_screens[i])
    text_plot(bquote(.(a)==.(b), where=list(a=var_names[[over[1]]], b=over1[i])), cex=cex.var_names, srt=over_srt[1])
  }
  close.screen(right_screens)
  
  # close outer
  close.screen(outer_screens)
  
  # close left screen if bar was made on right
  if (bar && same_scale) {
    close.screen(bar_screens[1])
  }
  
  # Return to original screen
  screen(begin_screen, new=FALSE)
  
  invisible()
}

if (F) {
  csa()
  cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])}, bar=F)
}

gg_make_multicolor_title <- function() {
  # https://stackoverflow.com/questions/49735290/ggplot2-color-individual-words-in-title
  t1 <- textGrob(expression("[" * phantom(bold("min")) * "=0," * phantom(bold("max")) * "=1]"),
                 x = 0.5, y = 1.1, gp = gpar(col = "black"))
  
  t2 <- textGrob(expression(phantom("[") * bold("min") * phantom("=0,max=1]")),
                 x = 0.5, y = 1.1, gp = gpar(col = "cyan"))
  
  t3 <- textGrob(expression(phantom("[min=0,") * bold("max") * phantom("=1]")),
                 x = 0.5, y = 1.1, gp = gpar(col = "magenta"))
  annotation_custom(grobTree(t1, t2, t3))
}

#' Create a contour plot from a grid of data
#' 
#' Makes filled contour plot with an optional sidebar, essentially filled.contour function.
#' This version uses the split.screen() function to add the sidebar if bar is TRUE.
#' By default it won't show the bar but will show the min and max values in the plot title
#' along with their colors.
#' Using this function will make other functions such as points() called afterwards not put points
#' where you expect. Pass anything you want added to the plot area to afterplotfunc
#' as a function to get it to work properly.
#' 
#' @param x  x values, must form grid with y. If not given, it is assumed to be from 0 to 1.
#' @param y  y values, must form grid with x. If not given, it is assumed to be from 0 to 1.
#' @param z  z values at grid locations
#' @param xlim  x limits for the plot.
#' @param ylim  y limits for the plot.
#' @param zlim  z limits for the plot.
#' @param with_lines Should lines be added on top of 
#' contour to show contours?
#' @param lines_only Should no fill be used, only contour lines?
#' @param levels  a set of levels which are used to partition the range of z. Must be strictly increasing (and finite). Areas with z values between consecutive levels are painted with the same color.
#' @param nlevels  if levels is not specified, the range of z, values is divided into approximately this many levels.
#' @param color.palette  A color palette function to be used to assign colors
#' in the plot. Defaults to cm.colors.strong. Other options include rainbow,
#' heat.colors, terrain.colors, topo.colors, and function(x) {gray((1:x)/x)}.
#' @param col  an explicit set of colors to be used in the plot. This argument overrides any palette function specification. There should be one less color than levels
#' @param plot.title  statements which add titles to the main plot.
#' @param plot.axes  statements which draw axes (and a box) on the main plot. This overrides the default axes.
#' @param key.title  statements which add titles for the plot key.
#' @param key.axes  statements which draw axes on the plot key. This overrides the default axis.
#' @param asp  the y/x aspect ratio, see plot.window.
#' @param xaxs  the x axis style. The default is to use internal labeling.
#' @param yaxs  the y axis style. The default is to use internal labeling.
#' @param las  the style of labeling to be used. The default is to use horizontal labeling.
#' @param axes  logical indicating if axes should be drawn, as in plot.default.
#' @param frame.plot  logical indicating if a box should be drawn, as in plot.default.
#' @param bar Should a bar showing the output range and colors be shown on the right?
#' @param pts Points to plot on top of contour
#' @param reset.par Should the graphical parameters be reset before exiting? Usually should be
#' unless you need to add something to the plot afterwards and bar is TRUE.
#' @param pretitle Text to be preappended to end of plot title
#' @param posttitle Text to be appended to end of plot title
#' @param main Title for the plot
#' @param mainminmax  whether the min and max values should be shown in the title of plot
#' @param mainminmax_minmax Whether [min,max]= should be shown in title or just the numbers
#' @param afterplotfunc Function to call after plotting, such as adding points or lines.
#' @param cex.main The size of the main title. 1.2 is default.
#' @param par.list List of options to pass to par
#' @param xaxis Should x axis be added?
#' @param yaxis Should y axis be added?
#' @param ...  additional graphical parameters, currently only passed to title().
#' @importFrom grDevices cm.colors
#' @importFrom graphics .filled.contour
#' @importFrom graphics Axis 
#' @importFrom graphics box
#' @importFrom graphics plot.new 
#' @importFrom graphics plot.window 
#' @importFrom graphics points 
#' @importFrom graphics title
#' @importFrom graphics par
#' @importFrom graphics axis layout lcm rect
#' @importFrom graphics split.screen screen close.screen
#' @examples 
#' x <- y <- seq(-4*pi, 4*pi, len = 27)
#' r <- sqrt(outer(x^2, y^2, "+"))
#' cf_grid(cos(r^2)*exp(-r/(2*pi)))
#' cf_grid(r, color.palette=heat.colors, bar=TRUE)
#' cf_grid(r, color.palette=function(x) {gray((1:x)/x)}, bar=TRUE)
#' @references
#' [1] filled.contour R function, copied function but removed part for sidebar
#' @references
#' [2] http://stackoverflow.com/questions/16774928/removing-part-of-a-graphic-in-r, answer by P Lapointe
#' @export
cf_grid <-
  function (x = seq(0, 1, length.out = nrow(z)), 
            y = seq(0, 1,length.out = ncol(z)), z, xlim = range(x, finite = TRUE),
            ylim = range(y, finite = TRUE), zlim = range(z, finite = TRUE),
            levels = pretty(zlim, nlevels), nlevels = 20, color.palette = cm.colors.strong,
            col = color.palette(length(levels) - 1), plot.title, plot.axes,
            key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1,
            axes = TRUE, frame.plot = axes, bar=F, pts=NULL, reset.par=TRUE,
            pretitle="", posttitle="", main=NULL,
            mainminmax=!bar, mainminmax_minmax=TRUE,
            afterplotfunc=NULL,
            cex.main=par()$cex.main,
            par.list=NULL,
            xaxis=TRUE, yaxis=TRUE,
            with_lines=FALSE, lines_only=FALSE,
            ...)
  {#browser()
    # filled.contour gives unnecessary legend, this function removes it
    # Used P Lapointe's solution from here: http://stackoverflow.com/questions/16774928/removing-part-of-a-graphic-in-r
    #   also had to changed .Internal(fillcontour) to .filled.contour
    #   and change layout to layout(matrix(c(1, 1), ncol = 1L), widths = c(1, lcm(w)))
    # Created 3/28/16 by Collin Erickson
    if (missing(z)) {
      if (!missing(x)) {
        if (is.list(x)) {
          z <- x$z
          y <- x$y
          x <- x$x
        } else {
          z <- x
          x <- seq.int(0, 1, length.out = nrow(z))
        }
      } else {
        stop("no 'z' matrix specified")
      }
    }
    else if (is.list(x)) {
      y <- x$y
      x <- x$x
    }
    if (any(diff(x) <= 0) || any(diff(y) <= 0))
      stop("increasing 'x' and 'y' values expected")
    
    if (any(diff(x) <= 0) || any(diff(y) <= 0)) 
      stop("increasing 'x' and 'y' values expected")
    # mar.orig <- (par.orig <- par(c("mar", "las", "mfrow")))$mar
    #if (reset.par) {on.exit({par(par.orig);close.screen(1)})}#all=TRUE)})}
    
    # This allows user to pass in par.list and it will return it after plotting
    par.names.to.save <- c("mar", "las", "mfrow", names(par.list))
    mar.orig <- (par.orig <- par(par.names.to.save))$mar
    if (!is.null(par.list)) {
      par(par.list)
    }
    
    #on.exit(close.screen(all=TRUE))
    w <- (3 + mar.orig[2L]) * par("csi") * 2.54
    #layout(matrix(c(if(bar) 2 else 1, 1), ncol = 2L), widths = c(1, lcm(w)))
    par(las = las)
    start.screen.number <- screen()
    if (bar) {#browser()
      #split.screen(c(1,2))
      screen.numbers <- split.screen(matrix(c(0,.85,0,1,.85,1,0,1), ncol=4, byrow=T))
      screen1 <- screen.numbers[1]
      screen2 <- screen.numbers[2]
      screen(screen2)
      mar <- mar.orig
      mar[4L] <- 2.5#mar[2L] # right
      mar[1] <- 2.2 # bottom
      mar[3] <- if (mainminmax | !is.null(main)) 1.3 else .3 #1.3#1.3 # top
      mar[2L] <- 0#1 # left
      par(mar = mar)
      plot.new()
      plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i", 
                  yaxs = "i")
      rect(0, levels[-length(levels)], 1, levels[-1L], col = col)
      if (missing(key.axes)) {
        if (axes) 
          axis(4)
      }
      else key.axes
      box()
      if (!missing(key.title))
        key.title
      mar <- mar.orig
      mar[4L] <- 1 # right # Why is this here?
      close.screen(screen2)
      screen(screen1)
      mar[1L] <- 2.2 # bottom
      mar[2L] <- 2.5 # left
      mar[3L] <- if (mainminmax | !is.null(main)) 1.3 else .3 #1.3# 1.3 # top
    }
    if (!bar) {
      # Using screen even with 1 screen to avoid error. Adding points after didn't show up properly
      screen.numbers <- split.screen(c(1,1)) 
      screen1 <- screen.numbers[1]
      screen(screen1)
      # Changing the margin to get bigger and square
      mar <- mar.orig #<- par()$mar
      mar[1] <- 2.2 # bottom
      mar[2] <- 2.5 # left
      mar[3] <- if (mainminmax | !is.null(main)) 1.3 else .3 # top
      mar[4] <- 1 # right
      
      # if (!missing(plot.axes)) {
      #   # TODO I shouldn't use plot.axes like this, FIX THIS
      #   mar[1] <- .3 # bottom
      #   mar[2] <- .3 # left
      #   mar[4] <- .3 # 1 # right
      # }
      if (!xaxis && !yaxis) {
        mar[1] <- .3 # bottom
        mar[2] <- .3 # left
        mar[3] <- if (mainminmax | !is.null(main)) 1.3 else .3 # top
        mar[4] <- .3 # right
      } else if (!xaxis) {
        mar[1] <- 1-.7 # bottom
        mar[3] <- if (mainminmax | !is.null(main)) 1.3 else .3 # top
        mar[4] <- .3 # right
      } else if (!yaxis) {
        mar[2] <- 1-.7 # left
        mar[3] <- if (mainminmax | !is.null(main)) 1.3 else .3 # top
        mar[4] <- .3 # right
      }
    }
    par(mar = mar)
    # par(cex.axis = 2)
    plot.new()
    plot.window(xlim, ylim, "", xaxs = xaxs, yaxs = yaxs, asp = asp)
    if (!is.matrix(z) || nrow(z) <= 1L || ncol(z) <= 1L)
      stop("no proper 'z' matrix specified")
    if (!is.double(z))
      storage.mode(z) <- "double"
    #.Internal(filledcontour(as.double(x), as.double(y), z, as.double(levels),
    #                        col = col))
    if (!lines_only) {
      .filled.contour(as.double(x), as.double(y), z, as.double(levels),
                      col = col)
    }
    # Something like this will remove axis numbers and ticks
    # Axis(x, side=1, labels=F, tick=F)
    if (missing(plot.axes)) {
      if (axes) {
        # title(main = "", xlab = "", ylab = "")
        if (xaxis) Axis(x, side = 1)
        if (yaxis) Axis(y, side = 2)
      }
    }
    else plot.axes
    if (frame.plot)
      box()
    #if (missing(plot.title))
    #  title(...)
    #else plot.title
    
    if (mainminmax | !is.null(main)) {
      make.multicolor.title(main=main, z=z, pretitle=pretitle,
                            posttitle=posttitle,
                            mainminmax_minmax=mainminmax_minmax,
                            col_min=col[1], col_max=col[length(col)],
                            cex.main=cex.main)
    }
    
    # Add contour lines if required
    if (with_lines || lines_only) {
      contour(x=x, y=y, z=z, add=T)
    }
    
    # Add points (pts) if given
    if (!is.null(pts)) {
      if (!is.matrix(pts)) { # if not a matrix, make it a matrix by row
        if (is.numeric(pts) && (length(pts)%%2==0)) {
          pts <- matrix(pts, ncol=2, byrow = T)
        }
      }
      points(pts, pch=19)
    }
    
    if (!is.null(afterplotfunc)) {
      afterplotfunc()
    }
    
    reset.par.func <- function() {
      if (T) {close.screen(screen1)}
      if (start.screen.number != FALSE) {screen(start.screen.number, new=FALSE)}
      par(par.orig)
    }
    if (reset.par) {# Either reset parameters
      reset.par.func()
      invisible()
    } else { # or return it so user can do it later
      return(reset.par.func)
    }
  }

#' Contour plot from data
#' 
#' Makes filled contour plot from data without sidebar by interpolating 
#' with a Gaussian process model.
#' The model is passed to cf_func to make the contour plot.
#' 
#' @param x  either just x data, x and y data, or x, y and z data
#' @param y  either y data, z data, or null
#' @param z  either z data or null
#' @param xlim  x limits for the contour plot, will be set to data limits +- 5\% if not specified
#' @param ylim  y limits for the contour plot, will be set to data limits +- 5\% if not specified
#' @param xylim x and y limits for the contour plot
#' @param fit Method to fit a model with. Current options are laGP (default)
#' and mlegp. laGP is faster but might cause trouble.
#' @param gg If TRUE, will use ggplot2 by calling gcf_func
#' @param ...  passed to cf_func
#' @importFrom utils capture.output
#' @examples 
#' x <- runif(20)
#' y <- runif(20)
#' z <- exp(-(x-.5)^2-5*(y-.5)^2)
#' cf_data(x,y,z)
#' @export
cf_data <- function(x, y=NULL, z=NULL,
                    xlim=NULL, ylim=NULL, xylim=NULL,
                    fit="",
                    gg=FALSE,
                    ...) {
  # Function that creates a contour plot from a data set
  # using a Gaussian process interpolation from mlegp
  #  x,y,z: three dimensional data, can be given only in x, in x and y, or in all three
  #  xlim,ylim: contour limits will be set to data limits +- 5% if not specified
  #  ... parameters passed to cf_func
  # Created 5/23/16 by Collin Erickson
  #require(mlegp)
  # This section parses data into x, y, and z
  if (is.null(y) & !is.null(z)) {
    if(dim(x)[2]!=2) {stop('Either give y or x must be matrix')}
    y <- x[,2]
    x <- x[,1]
  } else if (!is.null(y) & is.null(z)) {
    if(dim(x)[2]!=2) {stop('Either give y or x must be matrix')}
    z <- y
    y <- x[,2]
    x <- x[,1]
  } else if (is.null(y) & is.null(z)) {
    if (dim(x)[2]!=3) {stop('If only giving x it must have three columns')}
    z <- x[,3]
    y <- x[,2]
    x <- x[,1]
  }
  # Fits a Gaussian process model that interpolates perfectly, i.e., no smoothing
  if (fit == "mlegp") {
    co <- capture.output(mod <- mlegp::mlegp(X=data.frame(x,y),Z=z,verbose=0))
    pred.func <- function(xx) {mlegp::predict.gp(mod,xx)}
  } else {
    X <- data.frame(x, y)
    da <- laGP::darg(list(mle=TRUE), X=X)
    ga <- laGP::garg(list(mle=TRUE), y=z)
    mod1 <- laGP::newGPsep(X=X, Z=z, d=da$start, g=ga$start, dK = TRUE)
    laGP::jmleGPsep(gpsepi = mod1, drange=c(da$min, da$max),
                    grange=c(ga$min, ga$max),
                    dab=da$ab, gab=ga$ab, verb=0, maxit=1000)
    
    pred.func <- function(xx) {laGP::predGPsep(mod1, xx, lite=TRUE)$mean}
  }
  
  minx <- min(x);maxx <- max(x);miny <- min(y);maxy <- max(y)
  if (!is.null(xylim)) {xlim <- ylim <- xylim}
  if(is.null(xlim)) {xlim <- c(minx-.05*(maxx-minx),maxx+.05*(maxx-minx))}
  if(is.null(ylim)) {ylim <- c(miny-.05*(maxy-miny),maxy+.05*(maxy-miny))}
  # Passes prediction function to cf_func
  if (gg) {
    gcf_func(fn0 = pred.func,xlim=xlim,ylim=ylim, pts=cbind(x,y), batchmax=500, ...)
  } else {
    cf_func(fn0 = pred.func,xlim=xlim,ylim=ylim, pts=cbind(x,y), batchmax=500, ...)
  }
}

#' Makes filled contour plot from function 
#' 
#' A contour plot of the given function without sidebar by default.
#' It calls the function `cf_grid` to make the actual plot.
#' 
#' @param fn0  function to plot, first argument must be two-dimensional
#' @param n  number of points in each dimension
#' @param xlim  x limits for the contour plot
#' @param ylim  y limits for the contour plot
#' @param xylim  x and y limits for the contour plot, use when both are same
#' #@param mainminmax  whether the min and max values should be shown in the title of plot
#' @param batchmax  number of datapoints that can be computed at a time
#' @param out.col.name  if a column needs to be selected from the function, specify it
#' @param out.name Selects with a $ the name from output to be used, for lists and data frames
#' #@param pretitle Text to be preappended to end of plot title
#' #@param posttitle Text to be appended to end of plot title
#' #@param title Title for the plot
#' #@param mainminmax_minmax Whether [min,max]= should be shown in title or just the numbers
#' @param pts Points to plot on top of contour
#' @param gg Should ggplot2 be used? Will use gcf_grid() instead of cf_grid().
#' @param ...  Passed to cf_grid
#' @examples 
#' cf_func(function(x){x[1]*x[2]})
#' cf_func(function(x)(exp(-(x[1]-.5)^2-5*(x[2]-.5)^2)))
#' cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=TRUE)
#' cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=TRUE, mainminmax=TRUE)
#' cf_func(function(x)(exp(-(x[1]-.5)^2-5*(x[2]-.5)^2)), with_lines=TRUE)
#' @references
#' [1] filled.contour R function, copied function but removed part for sidebar
#' @references
#' [2] http://stackoverflow.com/questions/16774928/removing-part-of-a-graphic-in-r, answer by P Lapointe
#' @export
cf_func <- function(fn0, n=100,
                    xlim=c(0,1), ylim=c(0,1), xylim=NULL,
                    batchmax=1, out.col.name=NULL,
                    out.name=NULL,
                    pts=NULL,
                    gg=FALSE,
                    ...) {
  if(!is.null(out.col.name)) {
    fn <- function(xx){fn0(xx)[,out.col.name]}
  } else if (!is.null(out.name)) {
    fn <- function(xx){fn0(xx)[[out.name]]}
  } else {
    fn <- fn0
  }
  if (!is.null(xylim)) {xlim <- ylim <- xylim}
  x <- seq(xlim[1],xlim[2],length.out = n)
  y <- seq(ylim[1],ylim[2],length.out = n)
  z <- eval_over_grid_with_batch(x, y, fn, batchmax)
  
  if (gg) {
    gcf_grid(x,y,z, pts=pts, ...)
  } else {
    cf_grid(x,y,z, pts=pts, ...)
  }
}

# I think cm.colors is good but sometimes too light.

#' Strong version of cm.colors color palette
#' 
#' Altered version of cm.colors that uses
#' full saturation to get stronger colors.
#'
#' @param n Number of color groups
#' @param alpha Alpha level
#'
#' @return Character vector of colors
#' @export
#'
#' @examples
#' # Character string output
#' cm.colors.strong(5)
#' 
#' # Plot to show these
#' sl <- 21
#' sx <- seq(0,1,l=sl)
#' plot(sx,sin(2*pi*sx), cex=5, col=cm.colors.strong(sl), pch=19);points(sx,sin(2*pi*sx), cex=5)
#' plot(sx,sin(2*pi*sx), cex=5, col=cm.colors(sl),        pch=19);points(sx,sin(2*pi*sx), cex=5)
cm.colors.strong <- function (n, alpha = 1) {
  if ((n <- as.integer(n[1L])) > 0L) {
    even.n <- n%%2L == 0L
    k <- n%/%2L
    l1 <- k + 1L - even.n
    l2 <- n - k + even.n
    c(if (l1 > 0L) grDevices::hsv(h = 6/12, s = seq.int(0.99, ifelse(even.n, 
                                                         0.99/k, 0), length.out = l1), v = 1, alpha = alpha), 
      if (l2 > 1) grDevices::hsv(h = 10/12, s = seq.int(0, 0.99, length.out = l2)[-1L], 
                      v = 1, alpha = alpha))
  }
  else character()
}

#' Makes filled contour plot without sidebar, essentially filled.contour function.
#' This was the original version of cf_grid, before I tried adding the bar option.
#' I used it when I had trouble adding the bar to the function, but now that it works
#' I don't need this.
#' The legend part is commented out so `bar` does nothing.
#' Will no longer export since it doesn't do anything useful anymore.
#' @param x  x values, must form grid with y
#' @param y  y values, must form grid with x
#' @param z  z values at grid locations
#' @param xlim  x limits for the plot.
#' @param ylim  y limits for the plot.
#' @param zlim  z limits for the plot.
#' @param levels  a set of levels which are used to partition the range of z. Must be strictly increasing (and finite). Areas with z values between consecutive levels are painted with the same color.
#' @param nlevels  if levels is not specified, the range of z, values is divided into approximately this many levels.
#' @param color.palette  a color palette function to be used to assign colors in the plot.
#' @param col  an explicit set of colors to be used in the plot. This argument overrides any palette function specification. There should be one less color than levels
#' @param plot.title  statements which add titles to the main plot.
#' @param plot.axes  statements which draw axes (and a box) on the main plot. This overrides the default axes.
#' @param key.title  statements which add titles for the plot key.
#' @param key.axes  statements which draw axes on the plot key. This overrides the default axis.
#' @param asp  the y/x aspect ratio, see plot.window.
#' @param xaxs  the x axis style. The default is to use internal labeling.
#' @param yaxs  the y axis style. The default is to use internal labeling.
#' @param las  the style of labeling to be used. The default is to use horizontal labeling.
#' @param axes  logical indicating if axes should be drawn, as in plot.default.
#' @param frame.plot  logical indicating if a box should be drawn, as in plot.default.
#' @param bar Should a bar showing the output range and colors be shown on the right?
#' @param pts Points to plot on top of contour
#' @param reset.par Should the graphical parameters be reset before exiting? Usually should be.
#' @param ...  additional graphical parameters, currently only passed to title().
#' @importFrom grDevices cm.colors
#' @importFrom graphics .filled.contour
#' @importFrom graphics Axis 
#' @importFrom graphics box
#' @importFrom graphics plot.new 
#' @importFrom graphics plot.window 
#' @importFrom graphics points 
#' @importFrom graphics title
#' @importFrom graphics par
#' @examples 
#' x <- y <- seq(-4*pi, 4*pi, len = 27)
#' r <- sqrt(outer(x^2, y^2, "+"))
#' cf_grid2(cos(r^2)*exp(-r/(2*pi)))
#' @references
#' [1] filled.contour R function, copied function but removed part for sidebar
#' @references
#' [2] http://stackoverflow.com/questions/16774928/removing-part-of-a-graphic-in-r, answer by P Lapointe
#' @export
cf_grid2 <-
  function (x = seq(0, 1, length.out = nrow(z)), 
            y = seq(0, 1,length.out = ncol(z)), z, xlim = range(x, finite = TRUE),
            ylim = range(y, finite = TRUE), zlim = range(z, finite = TRUE),
            levels = pretty(zlim, nlevels), nlevels = 20, color.palette = cm.colors,
            col = color.palette(length(levels) - 1), plot.title, plot.axes,
            key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1,
            axes = TRUE, frame.plot = axes, bar=NULL, pts=NULL, reset.par=NULL,...)
  {
    # filled.contour gives unnecessary legend, this function removes it
    # Used P Lapointe's solution from here: http://stackoverflow.com/questions/16774928/removing-part-of-a-graphic-in-r
    #   also had to changed .Internal(fillcontour) to .filled.contour
    #   and change layout to layout(matrix(c(1, 1), ncol = 1L), widths = c(1, lcm(w)))
    # Created 3/28/16 by Collin Erickson
    if (missing(z)) {
      if (!missing(x)) {
        if (is.list(x)) {
          z <- x$z
          y <- x$y
          x <- x$x
        }
        else {
          z <- x
          x <- seq.int(0, 1, length.out = nrow(z))
        }
      }
      else stop("no 'z' matrix specified")
    }
    else if (is.list(x)) {
      y <- x$y
      x <- x$x
    }
    if (any(diff(x) <= 0) || any(diff(y) <= 0))
      stop("increasing 'x' and 'y' values expected")
    #mar.orig <- (par.orig <- par(c("mar", "las", "mfrow")))$mar
    #on.exit(par(par.orig))
    # Comment out everything that was for legend
    #w <- (3 + mar.orig[2L]) * par("csi") * 2.54
    #layout(matrix(c(1, 1), ncol = 1L), widths = c(1, lcm(w)))
    #par(las = las)
    #mar <- mar.orig
    #mar[4L] <- mar[2L]
    #mar[2L] <- 1
    #par(mar = mar)
    #plot.new()
    #plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i",
    #            yaxs = "i")
    #    rect(0, levels[-length(levels)], 1, levels[-1L], col = col)
    #    if (missing(key.axes)) {
    #        if (axes)
    #            axis(4)
    #    }
    #    else key.axes
    #    box()
    #if (!missing(key.title))
    #  key.title
    #mar <- mar.orig
    #mar[4L] <- 1
    #par(mar = mar)
    # Changing the margin to get bigger and square
    mar <- mar.orig <- par()$mar
    mar[1] <- 2.2 # bottom
    mar[2] <- 2.5 # left
    mar[3] <- 1.3 # top
    mar[4] <- 1 # right
    par(mar = mar)
    plot.new()
    plot.window(xlim, ylim, "", xaxs = xaxs, yaxs = yaxs, asp = asp)
    if (!is.matrix(z) || nrow(z) <= 1L || ncol(z) <= 1L)
      stop("no proper 'z' matrix specified")
    if (!is.double(z))
      storage.mode(z) <- "double"
    #.Internal(filledcontour(as.double(x), as.double(y), z, as.double(levels),
    #                        col = col))
    .filled.contour(as.double(x), as.double(y), z, as.double(levels),
                    col = col)
    if (missing(plot.axes)) {
      if (axes) {
        title(main = "", xlab = "", ylab = "")
        Axis(x, side = 1)
        Axis(y, side = 2)
      }
    }
    else plot.axes
    if (frame.plot)
      box()
    if (missing(plot.title))
      title(...)
    else plot.title
    
    if (!is.null(pts)) {
      points(pts, pch=19)
    }
    
    invisible()
  }

#' Makes filled contour plot from function 
#' 
#' A contour plot of the given function without sidebar by default.
#' It calls the function `cf_grid` to make the actual plot.
#' 
#' @param fn0  function to plot, first argument must be two-dimensional
#' @param n  number of points in each dimension
#' @param xlim  x limits for the contour plot
#' @param ylim  y limits for the contour plot
#' @param xylim  x and y limits for the contour plot, use when both are same
#' #@param mainminmax  whether the min and max values should be shown in the title of plot
#' @param batchmax  number of datapoints that can be computed at a time
#' @param out.col.name  if a column needs to be selected from the function, specify it
#' @param out.name Selects with a $ the name from output to be used, for lists and data frames
#' #@param pretitle Text to be preappended to end of plot title
#' #@param posttitle Text to be appended to end of plot title
#' #@param title Title for the plot
#' #@param mainminmax_minmax Whether [min,max]= should be shown in title or just the numbers
#' @param pts Points to plot on top of contour
#' @param ...  Passed to cf_grid
#' @examples 
#' gcf_func(function(x){x[1]*x[2]})
#' gcf_func(function(x)(exp(-(x[1]-.5)^2-5*(x[2]-.5)^2)))
#' gcf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=TRUE, color.palette=terrain.colors)
#' gcf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=TRUE, mainminmax=TRUE)
#' gcf_func(function(x)(exp(-(x[1]-.5)^2-5*(x[2]-.5)^2)))
#' @export
gcf_func <- function(fn0, n=100,
                    xlim=c(0,1), ylim=c(0,1), xylim=NULL,
                    batchmax=1, out.col.name=NULL,
                    out.name=NULL,
                    pts=NULL,
                    ...) {
  cf_func(fn0=fn0, n=n,
          xlim=xlim, ylim=ylim, xylim=xylim,
          batchmax=batchmax,
          out.col.name=out.col.name,
          out.name=out.name,
          pts=pts,
          gg=TRUE,
          ...)
  # if(!is.null(out.col.name)) {
  #   fn <- function(xx){fn0(xx)[,out.col.name]}
  # } else if (!is.null(out.name)) {
  #   fn <- function(xx){fn0(xx)[[out.name]]}
  # } else {
  #   fn <- fn0
  # }
  # if (!is.null(xylim)) {xlim <- ylim <- xylim}
  # x <- seq(xlim[1],xlim[2],length.out = n)
  # y <- seq(ylim[1],ylim[2],length.out = n)
  # z <- eval_over_grid_with_batch(x, y, fn, batchmax)
  # 
  # gcf_grid(x,y,z, pts=pts, ...)
}

#' Make a plot with only text
#' 
#' 
#' @param p Text to put on a plot
#' @param x x-value of center of text, defaults to center
#' @param y y-value of center of text, defaults to center
#' @param cex Size of text
#' @param ... Arguments passed to plot
#' @export
#' @importFrom graphics plot text
#'
#' @references ZNK's answer on https://stackoverflow.com/questions/19918985/r-plot-only-text, retrieved 5/25/2018
#' @examples 
#' text_plot("Useful?", cex=5)
text_plot <- function(p, x=0.5, y=0.5, cex=2, ...) {
  omar <- par()$mar
  par(mar = c(0,0,0,0))
  plot(c(0, 1), c(0, 1), ann = F, bty = 'n', type = 'n', xaxt = 'n', yaxt = 'n')
  # text(x = 0.5, y = 0.5, paste("The following is text that'll appear in a plot window.\n",
  #                              "As you can see, it's in the plot window\n",
  #                              "One might imagine useful informaiton here"), 
  #      cex = 1.6, col = "black")
  text(x = x, y = y, p, cex=cex, ...)
  par(mar=omar)
}

#' Make contour plot from data or function using ggplot2
#' 
#' Simpler function for making contours with cf package.
#' Won't give argument completion, so all must be specified
#'
#' @param ... Arguments to be passed to cf_func or cf_data based on 
#' data type of first argument. If D is given as argument, then it
#' is passed to cf_highdim.
#'
#' @return Whatever is returned from other function, probably nothing.
#' Will be a ggplot2 object if using gg=TRUE.
#' @export
#'
#' @examples
#' gcf(function(x){x[1]^2 - x[2]})
#' x <- runif(20)
#' y <- runif(20)
#' z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
#' gcf(x,y,z)
#' gcf(function(x){x[1]^2 - x[2]}, D=3)
gcf <- function(...) {
  cf(..., gg=TRUE)
}

#' Plot 2D contour slices of higher dimensional functions
#' 
#' Plots a grid of contour plots.
#' Each contour plot is a contour over two dimensions with the remaining
#' dimensions set to the baseline value.
#' Similar to plots created in Hwang et al. (2018).
#'
#' @param func Function to plot contours of
#' @param D Input dimension of function
#' @param low Low input value for each dimension
#' @param high High input value for each dimension
#' @param baseline Baseline input value for each dimension
#' @param n Number of points in grid on each dimension
#' @param batchmax  number of datapoints that can be computed at a time
#' @param same_scale Should all contour plots be on the same scale?
#' @param var_names Variable names to add to plot
#' Takes longer since it has to precalculate range of outputs.
#' @param pts Matrix of points to show on plot
#' @param average Should the background dimensions be averaged over instead of
#' set to baseline value? Much slower.
#' @param average_reps Number of points to average over when using average
#' @param key.title  statements which add titles for the plot key.
#' @param key.axes  statements which draw axes on the plot key. This overrides the default axis.
#' @param axes  logical indicating if axes should be drawn, as in plot.default.
#' @param nlevels  if levels is not specified, the range of z, values is
#' divided into approximately this many levels.
#' @param levels  a set of levels which are used to partition the range of z.
#' Must be strictly increasing (and finite). Areas with z values between
#' consecutive levels are painted with the same color.
#' @param color.palette  A color palette function to be used to assign colors
#' in the plot. Defaults to cm.colors.strong. Other options include rainbow,
#' heat.colors, terrain.colors, topo.colors, and function(x) {gray((1:x)/x)}.
#' @param col  an explicit set of colors to be used in the plot.
#' This argument overrides any palette function specification.
#' There should be one less color than levels.
#' @param bar Should a bar showing the output range and colors be shown on the top right?
#' @param edge_width How wide should edges with variable names be? As proportion of full screen.
#' @param cex.var_names Size of var_names printed on edges.
#' @param ... Arguments passed to cf_func, and then probably through to cf_grid
#'
#' @importFrom graphics contour mtext
#' @return NULL
#' @export
#' @references Hwang, Yongmoon, Sang-Lyul Cha, Sehoon Kim, Seung-Seop Jin,
#' and Hyung-Jo Jung. "The Multiple-Update-Infill Sampling Method Using
#' Minimum Energy Design for Sequential Surrogate Modeling."
#' Applied Sciences 8, no. 4 (2018): 481.
#'
#' @examples
#' \dontrun{
#' # Only use 4 dims of 8 for borehole function
#' cf_highdim(function(x) TestFunctions::borehole(c(x,.5,.5,.5,.5)), 4)
#' # Add points
#' cf_highdim(function(x) TestFunctions::borehole(c(x,.5,.5,.5,.5)), 4,
#'            pts=matrix(c(.1,.3,.6,.9),1,4))
#' 
#' # Full 8D borehole function
#' cf_highdim(TestFunctions::borehole, 8)
#' 
#' # Putting each plot on separate scale
#' cf_highdim(TestFunctions::borehole, 8, n=10, same_scale = FALSE)
#' }
#' 
#' cf_highdim(function(x) {x[1]^2 + exp(x[2])}, D=3)
#' 
#' friedman <- function(x) {
#'   10*sin(pi*x[1]*x[2]) + 20*(x[3]-.5)^2 + 10*x[4] + 5*x[5]
#' }
#' cf_highdim(friedman, 5, color.palette=topo.colors)
#' cf_highdim(friedman, 5, 
#'            color.palette=function(x) {gray((1:x)/x)},
#'            nlevels=10)
#'            
#' \dontrun{
#' # Recreate Plate 1 or Figure 1.1 from Engineering Design via Surrogate
#' # Modelling by Forrester, Sobester, and Keane (2008).
#' cf_highdim(function(x)TestFunctions::wingweight(x, scale_it=FALSE),
#'   D=10, low = c(150,220,6,-10,16,.5,.08,2.5,1700,.025),
#'   high = c(200,300,10,10,45,1,.18,6,2500,.08),
#'   baseline=c(174,252,7.52,0,34,.672,.12,3.8,2000,.064),
#'   color.palette=topo.colors, 
#'   var_names=c('SW', 'Wtw', 'A', 'Lambda', 'q', 'lambda', 'tc', 'Nz', 'Wdg'))
#' }
#' 
#' # Average over background dimensions, use higher reps to reduce noise.
#' f1 <- function(x) {x[1] + x[2]^2 + x[3]^3}
#' cf_highdim(f1, 4, average=TRUE, average_reps=1e2, n=10)
#' f1b <- function(x) {x[,1] + x[,2]^2 + x[,3]^3}
#' cf_highdim(f1b, 4, average=TRUE, average_reps=1e2, n=10, batchmax=Inf)
#' cf_highdim(f1b, 4, average_reps=1e2, n=10, batchmax=Inf,
#'            color.palette = topo.colors, nlevels=3)
#' 
#' # This was giving bad result
#' csa()
#' split.screen(c(2,1))
#' screen(2)
#' cf_highdim(f1b, 4, n=10, batchmax=Inf)
#' csa()
cf_highdim <- function(func, D, low=rep(0,D), high=rep(1,D),
                       baseline=(low+high)/2, same_scale=TRUE,
                       n=20,
                       batchmax=1,
                       # var_names=paste0("x",1:D),
                       var_names=c(expression(), 
                                   lapply(1:D,
                                          function(ti) bquote(x[.(ti)]))),
                       pts=NULL,
                       average=FALSE, average_reps=1e4,
                       axes=TRUE, key.axes, key.title,
                       nlevels=20, levels=pretty(zlim, nlevels),
                       color.palette=cm.colors.strong, 
                       col=color.palette(length(levels) - 1),
                       edge_width=.04, cex.var_names=1.3,
                       bar=TRUE,
                       ...) {
  # TODO put var_names in sep row/col
  # To put them all on same scale, need range of values first
  begin_screen <- screen()
  if (!is.null(pts)) {
    if (ncol(pts) != D) {stop("pts must have D columns")}
  }
  # Make version of function for just two dimensions
  
  if (average) {
    # Average over hidden dimensions
    funcij <- function(xx, i, j) {
      # There's a double batch issue: matrix of points to eval,
      #  then I use matrices for the average. So instead I'll
      #  force the first into vectors
      if (is.matrix(xx)) {return(apply(xx, 1, funcij, i=i, j=j))}
      ds <- c(i, j)
      notds <- setdiff(1:D, ds)
      XX <- lhs::randomLHS(average_reps, D-2)
      X4 <- matrix(nrow=average_reps, ncol=D)
      X4[, ds[1]] <- xx[1]
      X4[, ds[2]] <- xx[2]
      X4[, notds] <- XX
      if (batchmax > nrow(X4)) {
        mean(func(X4))
      } else if (batchmax > 1) { # Tricky case, need to split into groups
        sum(sapply(1:ceiling(nrow(X4)/batchmax),
               function(k) {
                 sum(
                   func(X4[(1+(k-1)*batchmax):min(nrow(X4), k*batchmax),])
                 )
               })) / nrow(X4)
      } else { # batchmax == 1
        mean(apply(X4, 1, func))
      }
    }
  } else {
    funcij <- function(xx, i, j) {
      if (batchmax == 1) {
        mid2 <- baseline
        mid2[i] <- xx[1]
        mid2[j] <- xx[2]
      } else { # batchmax > 1, use matrix
        mid2 <- matrix(baseline, nrow=nrow(xx), ncol=D, byrow=T)
        mid2[,i] <- xx[,1]
        mid2[,j] <- xx[,2]
      }
      func(mid2)
    }
  }
  # Get this function as a function
  get_funcij <- function(i,j) {
    function(x) {
      funcij(xx=x, i=i, j=j)
    }
  }
  if (same_scale) {
    # Put all plots on same scale, need to know max and min values before
    #  beginning plot, so it is twice as slow.
    zmin <- Inf
    zmax <- -Inf
    zlist <- list()
    for (j in 2:D) {
      y <- seq(low[j], high[j], length.out=n)
      zlist[[j]] <- list()
      for (i in 1:(j-1)) {
        x <- seq(low[i], high[i], length.out=n)
        zij <- eval_over_grid_with_batch(x, y, fn=get_funcij(i=i,j=j), batchmax)
        zlist[[j]][[i]] <- zij
        zmin <- min(zmin, min(zij))
        zmax <- max(zmax, max(zij))
      }
    }
    zlim <- c(zmin, zmax)
  }
  
  # outer_screens <- split.screen(c(2,2))
  # edge_width <- .06 # outer split point
  outer_screens <- split.screen(
    matrix(c(0,edge_width,edge_width,1,
             edge_width,1,edge_width,1,
             0,edge_width,0,edge_width,
             edge_width,1,0,edge_width), byrow=T, ncol=4))
  screen(outer_screens[2])
  
  # TODO change bar size, do separate split screen first, make it taller/wider depending on D  
  if (bar && same_scale) {
    # Make bar in top right square
    # Messed up labels when this was below plots, no clue why
    # screen(screen.numbers[D-1])
    bar_screens <- split.screen(matrix(c(3/4, 1, 2/3, 1), byrow=T, ncol=4))
    screen(bar_screens[1])
    # levels <- pretty(zlim, nlevels)
    # col <- color.palette(length(levels) - 1)
    okmar <- par()$mar
    kmar <- numeric(4) #mar.orig
    kmar[4L] <- 2.5#mar[2L] # right
    kmar[1] <- 2.2 # bottom
    kmar[3] <- .83 #if (mainminmax | !is.null(main)) 1.3 else .3 #1.3#1.3 # top
    kmar[2L] <- 3#0#1 # left
    par(mar = kmar)
    kmai <- par("mai")
    kdin <- par("din")
    # avail_left <- (1-edge_width) * kdin[1]/(D-1) - kmai[4]-.3 # When put into single box
    avail_left <- (1-edge_width) * kdin[1]/4 - kmai[4]-.3
    max_bar_width <- 0.5 # inches
    min_bar_width <- 0.1 # inches
    leftmai <- if (avail_left < min_bar_width) {0}
               else if (avail_left < min_bar_width + max_bar_width) {.1}
               else {avail_left - max_bar_width}
    
    kmai2 <- c(.1,
               # max(.5, min(par("mai")[2], par("din")[1]/2 - par("mai")[4]-0.3)),
               leftmai,
               .1,
               par("mai")[4])
    par(mai = kmai2)
    plot.new()
    plot.window(xlim = c(0, 1), ylim = range(levels), xaxs = "i", 
                yaxs = "i")
    rect(0, levels[-length(levels)], 1, levels[-1L], col = col)
    if (missing(key.axes)) {
      if (axes) 
        axis(4, las=1)
    }
    else key.axes
    box()
    if (!missing(key.title))
      key.title
    # mar <- mar.orig
    par(mar=okmar)
    close.screen(bar_screens)
    screen(outer_screens[2], new = FALSE)
  }
  
  # Split screen for grid of plots
  par(mar=c(1,1,1,1))
  screen.numbers <- split.screen(c(D-1, D-1), erase = FALSE)
  current_screen_index <- 1
  current_screen <- screen.numbers[current_screen_index]
  
  for (j in 2:D) {
    for (i in 1:(j-1)) {
      ds <- c(i, j)
      notds <- setdiff(1:D, ds)
      screen(current_screen)
      if (same_scale) {
        # Already calculated values, so pass them to cf_grid
        # cf_func(get_funcij(i=i,j=j), batchmax=batchmax,
        #         mainminmax=FALSE, plot.axes=F,
        #         xlim=c(low[i],high[i]), ylim=c(low[j],high[j]),
        #         zlim=zlim, pts=pts[,c(i,j)], ...)
        cf_grid(x=seq(low[i], high[i], length.out=n),
                y=seq(low[j], high[j], length.out=n),
                z=zlist[[j]][[i]],
                mainminmax=FALSE, xaxis=F&&(j==D), yaxis=F&&(i==1), #plot.axes=F,
                xlim=c(low[i],high[i]), ylim=c(low[j],high[j]),
                zlim=zlim, pts=pts[,c(i,j)],
                nlevels=nlevels, levels=levels,
                color.palette=color.palette, col=col,
                ...)
      } else {
        cf_func(get_funcij(i=i,j=j), batchmax=batchmax,
                mainminmax=FALSE, xaxis=F, yaxis=F, #plot.axes=F,
                xlim=c(low[i],high[i]), ylim=c(low[j],high[j]),
                pts=pts[,c(i,j)],
                ...)
      }
      current_screen_index <- current_screen_index + 1
      current_screen <- screen.numbers[current_screen_index]
    }
    if (j < D) {
      for (k in 1:(D - j)) {
        current_screen_index <- current_screen_index + 1
        current_screen <- screen.numbers[current_screen_index]
      }
    }
  }
  close.screen(n = screen.numbers)
  screen(outer_screens[1])
  left_screens <- split.screen(c(D-1, 1))
  for (j in 2:D) {
    screen(left_screens[j-1])
    text_plot(var_names[j], cex=cex.var_names)
  }
  close.screen(left_screens)
  screen(outer_screens[4])
  right_screens <- split.screen(c(1, D-1))
  for (i in 1:(D-1)) {
    screen(right_screens[i])
    text_plot(var_names[i], cex= cex.var_names)
  }
  close.screen(right_screens)
  
  # close outer
  close.screen(outer_screens)
  
  
  # Return to original screen
  screen(begin_screen, new=FALSE)
  
  # Add variable names
  # for (j in 2:D) {
  #   mtext(var_names[j], 2, at=(D-j+.5)/(D-1)*1.14-.07)
  # }
  # for (i in 1:(D-1)) {
  #   mtext(var_names[i], 1, at=(i-.5)/(D-1)*1.14-.07)
  # }
  invisible()
}
if (F) {
  close.screen(all.screens = TRUE)
  # cf_highdim(function(x) TestFunctions::borehole(c(x,.5,.5,.5,.5)), 4)
  cf_highdim(TestFunctions::borehole, 8)
}

# To get this to work, in cf_grid_screen, change Axis(x,1) part so that you can have it
#  only set one or other. Then only set values for y on left plots and x on right

make.multicolor.title <- function(main, z, pretitle, posttitle, mainminmax_minmax, col_min, col_max, cex.main=par()$cex.main) {
  if(is.null(main)) {
    title_text <- c(pretitle)
    title_color <- c(1)
    if (mainminmax_minmax) {
      title_text  <- c(title_text, '[','min',      ', ','max',      '] = ')
      title_color <- c(title_color,1,  col_min,1,   col_max,1)
    }
    title_text  <- c(title_text, "[",signif(min(z),3),', ',signif(max(z),3),']',posttitle)
    title_color <- c(title_color,1,  1,               1,   1,               1,  1)
    multicolor.title(title_text,title_color, cex.main=cex.main)
  } else {
    multicolor.title(main, 1, cex.main=cex.main)
  }
}

split.screen(c(1,2))
screen(1)
cf::cf(TestFunctions::banana,bar=T)
plot(1:10)
screen(2)
cf::cf(TestFunctions::branin, bar=F, mainminmax=F)
close.screen(2)
hist(runif(10))
close.screen(1)
hist(rnorm(70))
split.screen()
close.screen(all=T)



split.screen(c(1,3))
screen(2)
cf::cf(TestFunctions::banana,bar=T)
screen()

split.screen()
close.screen(all=T)

a <- b <- seq(-4*pi, 4*pi, len = 27)
r <- sqrt(outer(a^2, b^2, "+"))
cf_grid(cos(r^2)*exp(-r/(2*pi)))
cf_grid(cos(r^2)*exp(-r/(2*pi)), bar=T)
cf_grid(cos(r^2)*exp(-r/(2*pi)), bar=T, mainminmax = T)


x <- runif(20)
y <- runif(20)
z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
cf_data(x,y,z)
cf_data(x,y,z, bar=T)

cf_func(function(xx){exp(-sum((xx-.5)^2/.1))})
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=T)
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, main="Title check")
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=T, main="Title check")
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))},pts=matrix(c(.2,.2,.2,.8,.8,.2,.8,.8),byrow=T,ncol=2))
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))})
points(matrix(c(.2,.2,.2,.8,.8,.2,.8,.8),byrow=T,ncol=2)) #These should show up at right spot


split.screen(c(2,2))
screen(1)
cf_grid3(cos(r^2)*exp(-r/(2*pi)), bar=T)
close.screen(1)
screen(2)
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=T)
close.screen(2)
close.screen(all=T)

# Checking text size
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=F)
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=F, cex.main=.5)
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=F, cex=3)

# Checking par.list, everything should go back to normal after each plot
cf_grid(cos(r^2)*exp(-r/(2*pi)))
cf_grid(cos(r^2)*exp(-r/(2*pi)), par.list=list(col='red', cex.axis=2.5))
cf_grid(cos(r^2)*exp(-r/(2*pi)))
cf_grid(cos(r^2)*exp(-r/(2*pi)), par.list=list(col='red', cex.axis=2.5, lwd=5))
cf_grid(cos(r^2)*exp(-r/(2*pi)))
plot(1:10)
cf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=F, par.list=list(col='red', cex.axis=2.5, lwd=5))
cf_grid(cos(r^2)*exp(-r/(2*pi)))

#' Evaluate function over grid of points
#' 
#' `batchmax` gives how many can be evaluated at a time.
#' If more than 1, then the input is given to the function
#' as rows of a matrix.
#' 
#'
#' @param x Vector of x values to evaluate
#' @param y Vector of y values to evaluate
#' @param fn Function that takes in a length two vector if `batchmax` is 1
#' or a matrix with two columns if greater than 1.
#' @param batchmax Number of points that can evaluated simultaneously.
#' If 1, points are passed to `fn` as a vector of length two.
#' If greater than 1, points are passed to `fn` as rows of a matrix.
#'
#' @return Matrix of size `length(x)` by `length(y)`
#' @export
#'
#' @examples
#' eval_over_grid_with_batch(c(0,.5,1), c(10,20,30), function(a)a[1]+a[2], batchmax=1)
#' eval_over_grid_with_batch(c(0,.5,1), c(10,20,30), function(a)a[,1]+a[,2], batchmax=Inf)
eval_over_grid_with_batch <- function(x, y, fn, batchmax) {
  nx <- length(x)
  ny <- length(y)
  if(batchmax<=1) { # calculate single Z value at a time
    #for(xi in 1:n) for(yi in 1:n) {z[xi,yi] <- fn(c(x[xi],y[yi]))}
    fn_outer <- Vectorize(function(xi, yi) {fn(c(x[xi], y[yi]))})
    z <- outer(1:nx, 1:ny, fn_outer)
  } else {
    z <- matrix(NA,nx,ny)
    inbatch <- 0
    for(xi in 1:nx) {
      for(yi in 1:ny) {
        if(inbatch==0) XYi <- matrix(c(xi,yi),ncol=2)
        else XYi <- rbind(XYi,matrix(c(xi,yi),ncol=2))
        inbatch <- inbatch + 1
        if(inbatch == batchmax | (xi==nx & yi==ny)) {
          Zbatch <- fn(matrix(c(x[XYi[,1]],y[XYi[,2]]),ncol=2,byrow=F))
          for(rowbatch in 1:length(Zbatch)) {
            z[XYi[rowbatch,1],XYi[rowbatch,2]] <- Zbatch[rowbatch]
          }
          inbatch <- 0
          rm(XYi)
        }
      }
    }
  }
  z
}

I added new functions and new functionality.

## Test environments
* local Windows 7 install, R 3.6.0
* ubuntu 14.04 (on travis-ci), R 3.6.0
* win-builder (devel and release)

## R CMD check results

0 errors | 0 warnings | 0 notes

## Reverse dependencies

The only reverse dependency is another package of mine,
and it doesn't affect it.

---
title: "Introduction to the ContourFunctions R package"
author: "Collin Erickson"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to the ContourFunctions R package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(fig.width=6, fig.height=6)
```


The ContourFunctions R package provides functions that make it easier to make contour plots. The function `cf` is a quick function that can take in grid data, a function, or any data, and give a contour plot showing the function or data.
By default, plots are made using base graphics,
but they can also be done using ggplot2.

## `cf_grid`

`cf_grid` creates a contour plot from a grid of points.

Below `a` and `b` create a grid of points at which `r` is calculated. `cf_grid` is used to create the contour plot.
Note that the only indication of the relationship between the colors and the `r` values is in the title of plot, which says that the darkest blue point is the minimum of -0.613, and the darkest pink point is the maximum of 1. (Note that this is not a good representation of the surface because there aren't enough points in the grid, the contours are actually concentric circles as shown below.)

```{r cf_grid}
library(ContourFunctions)
a <- b <- seq(-4*pi, 4*pi, len = 27)
r <- sqrt(outer(a^2, b^2, "+"))
cf_grid(a, b, cos(r^2)*exp(-r/(2*pi)))
```

To add a bar that shows how the colors relate to the output, simply set `bar=TRUE`, as shown below.

```{r cf_grid bar}

cf_grid(a, b, cos(r^2)*exp(-r/(2*pi)), bar=TRUE)
```

Other parameters specifying details of the plot can be passed as well, see 
the documentation for those options.


## `cf_func`

For the above we had to create the grid of points and give it in to `cf_grid`. To make this easier, `cf_func` allows you to simply pass in a function. It will then evaluate the function at a grid of points and pass these to `cf_grid` to make the contour plot.

```{r}
f1 <- function(r) cos(r[1]^2 + r[2]^2)*exp(-sqrt(r[1]^2 + r[2]^2)/(2*pi))
cf_func(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi))
```


If you give a function that can more efficient evaluate a bunch of points at a time, instead of one at a time, use the `batchmax` to have it pass points as a matrix to the given function.

The argument `n` controls how many points along each dimension are used. We see below that if we go back to `n=27`, then we get the same plot as above.


```{r}
cf_func(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi), n=27)
```



## `cf_data`

Often one has data and wants to get an idea of what the surface looks like that fits the data. The `cf_data` allows the user to pass in the data to get such a plot. A Gaussian process model is fit to the data, by default using the R package laGP to do so. The model is then used to make predictions at the grid of points to make the contour plot. The model prediction function is passed to `cf_func` to create the contour plot.
Note that this relies heavily on the model being somewhat accurate, and may not truly represent the data if the model is a poor fit.

Below a random sample of 20 points are taken from a function (a Gaussian peak centered at (0.5, 0.5)), and `cf_data` is used to plot the data. The black dots show the data points used to create the model.

```{r cf_data}
set.seed(0)
x <- runif(20)
y <- runif(20)
z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
# cf_data(x,y,z)
cf_data(x,y,z, bar=T)
```



## `afterplotfunc`

The contour plots are created using the `split.screen` function.
This causes the plot to not add additional items, such as points or lines, after making the plot. The plot below shows how when trying to add a point to the plot using `points`, a point that should be placed at the center ends up in the bottom right corner.


```{r}
cf_func(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi))
points(c(0,0), pch=19)
```

If you just want to add points, you can use the parameter `pts` to do so. Below we see that the point ends up correctly in the center of the plot.

```{r}
cf_func(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi), pts=c(0,0))
```

Another option, that gives you more capability, is to use the parameter `afterplotfunc` to pass in a function that takes no arguments. After the plot is made this function will be called. You can put anything inside this function that you would normally do to a plot, including `points`, `text`, `legend`, and `abline`.



```{r}
cf_func(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi),
        afterplotfunc=function() {
          points(5, 5, pch=19)
          text(-5,5,"Text here")
          legend('bottomright', legend=c(1,2,3), fill=c(1,2,3))
          abline(a=0, b=1, col=2)
          }
        )
```



## `cf`

To make using the above `cf_func` and `cf_data` slightly easier, the same inputs can be passed to the function `cf`. It detects whether the first parameter is a function, in which case it passes everything to `cf_func` or numeric, in which case it passes everything to `cf_data`.

The following two plots demonstrate how `cf` is used. Really the only benefit is that is saves you typing `_func` or `_grid`.

```{r cf for func}
cf(f1, xlim = c(-4*pi, 4*pi), ylim = c(-4*pi, 4*pi))
```

```{r cf for data}
cf(x,y,z, bar=T)
```


## `cf_highdim`

For higher dimensional functions, `cf_highdim` makes a contour plot
of two-dimensional slices of the given function.
The dimensions not being shown can be set to a default value
or averaged out.

```{r}

friedman <- function(x) {
  10*sin(pi*x[1]*x[2]) + 20*(x[3]-.5)^2 + 10*x[4] + 5*x[5]
}
cf_highdim(friedman, 5, color.palette=topo.colors)
```


## `cf_4dim`

Functions with four input dimensions can be displayed
using a grid of contour plots with the function `cf_4dim`.
Two of the dimensions are shown on each plot,
while the other two are set to a specific value
for the given plot.


```{r}
cf_4dim(function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])})
```

## Making plots with ggplot2

All of the above plots used R base graphics.
Similar functions for `cf`, `cf_func`, `cf_data`, and `cf_grid`
that use ggplot2 are also available as
`gcf`, `gcf_func`, `gcf_data`, and `gcf_grid`

```{r}
f2 <- function(x) {exp(x[1]) * sin(2*pi*x[2])}
gcf(f2)
```


## Adding contour lines to plots

By default, the contour plots are made using filled colors.
To add lines on top of the color fill, use `with_lines=TRUE`.
To make a contour plot with only lines, use `lines_only=TRUE`.

```{r}
cf(f2, with_lines=TRUE)
```

```{r}
gcf(f2, lines_only=TRUE, bar=T)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/text_plot.R
\name{text_plot}
\alias{text_plot}
\title{Make a plot with only text}
\usage{
text_plot(p, x = 0.5, y = 0.5, cex = 2, ...)
}
\arguments{
\item{p}{Text to put on a plot}

\item{x}{x-value of center of text, defaults to center}

\item{y}{y-value of center of text, defaults to center}

\item{cex}{Size of text}

\item{...}{Arguments passed to plot}
}
\description{
Make a plot with only text
}
\examples{
text_plot("Useful?", cex=5)
}
\references{
ZNK's answer on https://stackoverflow.com/questions/19918985/r-plot-only-text, retrieved 5/25/2018
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cf_highdim.R
\name{cf_highdim}
\alias{cf_highdim}
\title{Plot 2D contour slices of higher dimensional functions}
\usage{
cf_highdim(func, D, low = rep(0, D), high = rep(1, D),
  baseline = (low + high)/2, same_scale = TRUE, n = 20,
  batchmax = 1, var_names = c(expression(), lapply(1:D, function(ti)
  bquote(x[.(ti)]))), pts = NULL, average = FALSE,
  average_reps = 10000, axes = TRUE, key.axes, key.title,
  nlevels = 20, levels = pretty(zlim, nlevels),
  color.palette = cm.colors.strong, col = color.palette(length(levels)
  - 1), edge_width = 0.04, cex.var_names = 1.3, bar = TRUE, ...)
}
\arguments{
\item{func}{Function to plot contours of}

\item{D}{Input dimension of function}

\item{low}{Low input value for each dimension}

\item{high}{High input value for each dimension}

\item{baseline}{Baseline input value for each dimension}

\item{same_scale}{Should all contour plots be on the same scale?}

\item{n}{Number of points in grid on each dimension}

\item{batchmax}{number of datapoints that can be computed at a time}

\item{var_names}{Variable names to add to plot
Takes longer since it has to precalculate range of outputs.}

\item{pts}{Matrix of points to show on plot}

\item{average}{Should the background dimensions be averaged over instead of
set to baseline value? Much slower.}

\item{average_reps}{Number of points to average over when using average}

\item{axes}{logical indicating if axes should be drawn, as in plot.default.}

\item{key.axes}{statements which draw axes on the plot key. This overrides the default axis.}

\item{key.title}{statements which add titles for the plot key.}

\item{nlevels}{if levels is not specified, the range of z, values is
divided into approximately this many levels.}

\item{levels}{a set of levels which are used to partition the range of z.
Must be strictly increasing (and finite). Areas with z values between
consecutive levels are painted with the same color.}

\item{color.palette}{A color palette function to be used to assign colors
in the plot. Defaults to cm.colors.strong. Other options include rainbow,
heat.colors, terrain.colors, topo.colors, and function(x) {gray((1:x)/x)}.}

\item{col}{an explicit set of colors to be used in the plot.
This argument overrides any palette function specification.
There should be one less color than levels.}

\item{edge_width}{How wide should edges with variable names be? As proportion of full screen.}

\item{cex.var_names}{Size of var_names printed on edges.}

\item{bar}{Should a bar showing the output range and colors be shown on the top right?}

\item{...}{Arguments passed to cf_func, and then probably through to cf_grid}
}
\description{
Plots a grid of contour plots.
Each contour plot is a contour over two dimensions with the remaining
dimensions set to the baseline value.
Similar to plots created in Hwang et al. (2018).
}
\examples{
\dontrun{
# Only use 4 dims of 8 for borehole function
cf_highdim(function(x) TestFunctions::borehole(c(x,.5,.5,.5,.5)), 4)
# Add points
cf_highdim(function(x) TestFunctions::borehole(c(x,.5,.5,.5,.5)), 4,
           pts=matrix(c(.1,.3,.6,.9),1,4))

# Full 8D borehole function
cf_highdim(TestFunctions::borehole, 8)

# Putting each plot on separate scale
cf_highdim(TestFunctions::borehole, 8, n=10, same_scale = FALSE)
}

cf_highdim(function(x) {x[1]^2 + exp(x[2])}, D=3)

friedman <- function(x) {
  10*sin(pi*x[1]*x[2]) + 20*(x[3]-.5)^2 + 10*x[4] + 5*x[5]
}
cf_highdim(friedman, 5, color.palette=topo.colors)
cf_highdim(friedman, 5, 
           color.palette=function(x) {gray((1:x)/x)},
           nlevels=10)
           
\dontrun{
# Recreate Plate 1 or Figure 1.1 from Engineering Design via Surrogate
# Modelling by Forrester, Sobester, and Keane (2008).
cf_highdim(function(x)TestFunctions::wingweight(x, scale_it=FALSE),
  D=10, low = c(150,220,6,-10,16,.5,.08,2.5,1700,.025),
  high = c(200,300,10,10,45,1,.18,6,2500,.08),
  baseline=c(174,252,7.52,0,34,.672,.12,3.8,2000,.064),
  color.palette=topo.colors, 
  var_names=c('SW', 'Wtw', 'A', 'Lambda', 'q', 'lambda', 'tc', 'Nz', 'Wdg'))
}

# Average over background dimensions, use higher reps to reduce noise.
f1 <- function(x) {x[1] + x[2]^2 + x[3]^3}
cf_highdim(f1, 4, average=TRUE, average_reps=1e2, n=10)
f1b <- function(x) {x[,1] + x[,2]^2 + x[,3]^3}
cf_highdim(f1b, 4, average=TRUE, average_reps=1e2, n=10, batchmax=Inf)
cf_highdim(f1b, 4, average_reps=1e2, n=10, batchmax=Inf,
           color.palette = topo.colors, nlevels=3)

# This was giving bad result
csa()
split.screen(c(2,1))
screen(2)
cf_highdim(f1b, 4, n=10, batchmax=Inf)
csa()
}
\references{
Hwang, Yongmoon, Sang-Lyul Cha, Sehoon Kim, Seung-Seop Jin,
and Hyung-Jo Jung. "The Multiple-Update-Infill Sampling Method Using
Minimum Energy Design for Sequential Surrogate Modeling."
Applied Sciences 8, no. 4 (2018): 481.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cm_colors_strong.R
\name{cm.colors.strong}
\alias{cm.colors.strong}
\title{Strong version of cm.colors color palette}
\usage{
cm.colors.strong(n, alpha = 1)
}
\arguments{
\item{n}{Number of color groups}

\item{alpha}{Alpha level}
}
\value{
Character vector of colors
}
\description{
Altered version of cm.colors that uses
full saturation to get stronger colors.
}
\examples{
# Character string output
cm.colors.strong(5)

# Plot to show these
sl <- 21
sx <- seq(0,1,l=sl)
plot(sx,sin(2*pi*sx), cex=5, col=cm.colors.strong(sl), pch=19);points(sx,sin(2*pi*sx), cex=5)
plot(sx,sin(2*pi*sx), cex=5, col=cm.colors(sl),        pch=19);points(sx,sin(2*pi*sx), cex=5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cf_csa.R
\name{csa}
\alias{csa}
\title{Close all open screens}
\usage{
csa(silent = FALSE)
}
\arguments{
\item{silent}{Should the output of `close.screen` not be returned?}
}
\description{
Closes the screens open, which happens
when plotting with `split.screen` is interrupted.
It often happens when there is a error while plotting.
When you try to plot
the next thing it gives an error.
Running this function will reset the plot screen.
It just does `close.screen(all.screens=TRUE)` but is faster to type.
}
\examples{
# Split screen into fourths
split.screen(c(2,2))
hist(rnorm(100))
screen(2)
hist(runif(100))
# Use csa() to go back to normal plotting
csa()
hist(rexp(100))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cf_data.R
\name{cf_data}
\alias{cf_data}
\title{Contour plot from data}
\usage{
cf_data(x, y = NULL, z = NULL, xlim = NULL, ylim = NULL,
  xylim = NULL, fit = "", gg = FALSE, ...)
}
\arguments{
\item{x}{either just x data, x and y data, or x, y and z data}

\item{y}{either y data, z data, or null}

\item{z}{either z data or null}

\item{xlim}{x limits for the contour plot, will be set to data limits +- 5\% if not specified}

\item{ylim}{y limits for the contour plot, will be set to data limits +- 5\% if not specified}

\item{xylim}{x and y limits for the contour plot}

\item{fit}{Method to fit a model with. Current options are laGP (default)
and mlegp. laGP is faster but might cause trouble.}

\item{gg}{If TRUE, will use ggplot2 by calling gcf_func}

\item{...}{passed to cf_func}
}
\description{
Makes filled contour plot from data without sidebar by interpolating 
with a Gaussian process model.
The model is passed to cf_func to make the contour plot.
}
\examples{
x <- runif(20)
y <- runif(20)
z <- exp(-(x-.5)^2-5*(y-.5)^2)
cf_data(x,y,z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eval_over_grid_with_batch.R
\name{eval_over_grid_with_batch}
\alias{eval_over_grid_with_batch}
\title{Evaluate function over grid of points}
\usage{
eval_over_grid_with_batch(x, y, fn, batchmax)
}
\arguments{
\item{x}{Vector of x values to evaluate}

\item{y}{Vector of y values to evaluate}

\item{fn}{Function that takes in a length two vector if `batchmax` is 1
or a matrix with two columns if greater than 1.}

\item{batchmax}{Number of points that can evaluated simultaneously.
If 1, points are passed to `fn` as a vector of length two.
If greater than 1, points are passed to `fn` as rows of a matrix.}
}
\value{
Matrix of size `length(x)` by `length(y)`
}
\description{
`batchmax` gives how many can be evaluated at a time.
If more than 1, then the input is given to the function
as rows of a matrix.
}
\examples{
eval_over_grid_with_batch(c(0,.5,1), c(10,20,30), function(a)a[1]+a[2], batchmax=1)
eval_over_grid_with_batch(c(0,.5,1), c(10,20,30), function(a)a[,1]+a[,2], batchmax=Inf)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cf_grid_screen.R
\name{cf_grid}
\alias{cf_grid}
\title{Create a contour plot from a grid of data}
\usage{
cf_grid(x = seq(0, 1, length.out = nrow(z)), y = seq(0, 1, length.out =
  ncol(z)), z, xlim = range(x, finite = TRUE), ylim = range(y, finite =
  TRUE), zlim = range(z, finite = TRUE), levels = pretty(zlim,
  nlevels), nlevels = 20, color.palette = cm.colors.strong,
  col = color.palette(length(levels) - 1), plot.title, plot.axes,
  key.title, key.axes, asp = NA, xaxs = "i", yaxs = "i", las = 1,
  axes = TRUE, frame.plot = axes, bar = F, pts = NULL,
  reset.par = TRUE, pretitle = "", posttitle = "", main = NULL,
  mainminmax = !bar, mainminmax_minmax = TRUE, afterplotfunc = NULL,
  cex.main = par()$cex.main, par.list = NULL, xaxis = TRUE,
  yaxis = TRUE, with_lines = FALSE, lines_only = FALSE, ...)
}
\arguments{
\item{x}{x values, must form grid with y. If not given, it is assumed to be from 0 to 1.}

\item{y}{y values, must form grid with x. If not given, it is assumed to be from 0 to 1.}

\item{z}{z values at grid locations}

\item{xlim}{x limits for the plot.}

\item{ylim}{y limits for the plot.}

\item{zlim}{z limits for the plot.}

\item{levels}{a set of levels which are used to partition the range of z. Must be strictly increasing (and finite). Areas with z values between consecutive levels are painted with the same color.}

\item{nlevels}{if levels is not specified, the range of z, values is divided into approximately this many levels.}

\item{color.palette}{A color palette function to be used to assign colors
in the plot. Defaults to cm.colors.strong. Other options include rainbow,
heat.colors, terrain.colors, topo.colors, and function(x) {gray((1:x)/x)}.}

\item{col}{an explicit set of colors to be used in the plot. This argument overrides any palette function specification. There should be one less color than levels}

\item{plot.title}{statements which add titles to the main plot.}

\item{plot.axes}{statements which draw axes (and a box) on the main plot. This overrides the default axes.}

\item{key.title}{statements which add titles for the plot key.}

\item{key.axes}{statements which draw axes on the plot key. This overrides the default axis.}

\item{asp}{the y/x aspect ratio, see plot.window.}

\item{xaxs}{the x axis style. The default is to use internal labeling.}

\item{yaxs}{the y axis style. The default is to use internal labeling.}

\item{las}{the style of labeling to be used. The default is to use horizontal labeling.}

\item{axes}{logical indicating if axes should be drawn, as in plot.default.}

\item{frame.plot}{logical indicating if a box should be drawn, as in plot.default.}

\item{bar}{Should a bar showing the output range and colors be shown on the right?}

\item{pts}{Points to plot on top of contour}

\item{reset.par}{Should the graphical parameters be reset before exiting? Usually should be
unless you need to add something to the plot afterwards and bar is TRUE.}

\item{pretitle}{Text to be preappended to end of plot title}

\item{posttitle}{Text to be appended to end of plot title}

\item{main}{Title for the plot}

\item{mainminmax}{whether the min and max values should be shown in the title of plot}

\item{mainminmax_minmax}{Whether [min,max]= should be shown in title or just the numbers}

\item{afterplotfunc}{Function to call after plotting, such as adding points or lines.}

\item{cex.main}{The size of the main title. 1.2 is default.}

\item{par.list}{List of options to pass to par}

\item{xaxis}{Should x axis be added?}

\item{yaxis}{Should y axis be added?}

\item{with_lines}{Should lines be added on top of 
contour to show contours?}

\item{lines_only}{Should no fill be used, only contour lines?}

\item{...}{additional graphical parameters, currently only passed to title().}
}
\description{
Makes filled contour plot with an optional sidebar, essentially filled.contour function.
This version uses the split.screen() function to add the sidebar if bar is TRUE.
By default it won't show the bar but will show the min and max values in the plot title
along with their colors.
Using this function will make other functions such as points() called afterwards not put points
where you expect. Pass anything you want added to the plot area to afterplotfunc
as a function to get it to work properly.
}
\examples{
x <- y <- seq(-4*pi, 4*pi, len = 27)
r <- sqrt(outer(x^2, y^2, "+"))
cf_grid(cos(r^2)*exp(-r/(2*pi)))
cf_grid(r, color.palette=heat.colors, bar=TRUE)
cf_grid(r, color.palette=function(x) {gray((1:x)/x)}, bar=TRUE)
}
\references{
[1] filled.contour R function, copied function but removed part for sidebar

[2] http://stackoverflow.com/questions/16774928/removing-part-of-a-graphic-in-r, answer by P Lapointe
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcf_grid.R
\name{gcf_grid}
\alias{gcf_grid}
\title{Create contour plot from grid data using ggplot2}
\usage{
gcf_grid(x = seq(0, 1, length.out = nrow(z)), y = seq(0, 1, length.out
  = ncol(z)), z, xlim = range(x, finite = TRUE), ylim = range(y, finite
  = TRUE), zlim = range(z, finite = TRUE), with_lines = FALSE,
  lines_only = FALSE, bins = 8, interpolate = TRUE,
  levels = pretty(zlim, nlevels), nlevels = 20,
  color.palette = cm.colors.strong, col = color.palette(length(levels)
  - 1), asp = NA, las = 1, bar = F, pts = NULL, reset.par = TRUE,
  pretitle = "", posttitle = "", main = NULL, mainminmax = !bar,
  mainminmax_minmax = TRUE, afterplotfunc = NULL,
  cex.main = par()$cex.main, ...)
}
\arguments{
\item{x}{x values, must form grid with y.
If not given, it is assumed to be from 0 to 1.}

\item{y}{y values, must form grid with x.
If not given, it is assumed to be from 0 to 1.}

\item{z}{z values at grid locations}

\item{xlim}{x limits for the plot.}

\item{ylim}{y limits for the plot.}

\item{zlim}{z limits for the plot.}

\item{with_lines}{Should lines be added on top of 
contour to show contours?}

\item{lines_only}{Should no fill be used, only contour lines?}

\item{bins}{Number of lines used when using `with_lines` or `lines_only`}

\item{interpolate}{Will smooth out contours}

\item{levels}{a set of levels which are used to partition the range of z.
Must be strictly increasing (and finite). Areas with z values between
consecutive levels are painted with the same color.}

\item{nlevels}{if levels is not specified, the range of z, values is
divided into approximately this many levels.}

\item{color.palette}{a color palette function to be used to assign colors
in the plot. Defaults to cm.colors. Other options include rainbow,
heat.colors, terrain.colors, topo.colors, and function(x) {gray((1:x)/x)}.}

\item{col}{an explicit set of colors to be used in the plot. This argument
overrides any palette function specification. There should be one less
color than levels}

\item{asp}{the y/x aspect ratio, see plot.window.}

\item{las}{the style of labeling to be used. The default is to use
horizontal labeling.}

\item{bar}{Should a bar showing the output range and colors be shown on the right?}

\item{pts}{Points to plot on top of contour}

\item{reset.par}{Should the graphical parameters be reset before exiting?
Usually should be unless you need to add something to the plot
afterwards and bar is TRUE.}

\item{pretitle}{Text to be preappended to end of plot title}

\item{posttitle}{Text to be appended to end of plot title}

\item{main}{Title for the plot}

\item{mainminmax}{whether the min and max values should be shown
in the title of plot}

\item{mainminmax_minmax}{Whether [min,max]= should be shown in title
or just the numbers}

\item{afterplotfunc}{Function to call after plotting, such as adding
points or lines.}

\item{cex.main}{The size of the main title. 1.2 is default.}

\item{...}{additional graphical parameters, currently only passed to title().}
}
\value{
ggplot2 object
}
\description{
The same as cf_grid_screen but uses ggplot2 for the plot.
}
\examples{
x <- y <- seq(-4*pi, 4*pi, len = 27)
r <- sqrt(outer(x^2, y^2, "+"))
gcf_grid(cos(r^2)*exp(-r/(2*pi)))
gcf_grid(r, color.palette=heat.colors, bar=TRUE)
gcf_grid(r, color.palette=function(x) {gray((1:x)/x)}, bar=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcf_func.R
\name{gcf_func}
\alias{gcf_func}
\title{Makes filled contour plot from function}
\usage{
gcf_func(fn0, n = 100, xlim = c(0, 1), ylim = c(0, 1),
  xylim = NULL, batchmax = 1, out.col.name = NULL, out.name = NULL,
  pts = NULL, ...)
}
\arguments{
\item{fn0}{function to plot, first argument must be two-dimensional}

\item{n}{number of points in each dimension}

\item{xlim}{x limits for the contour plot}

\item{ylim}{y limits for the contour plot}

\item{xylim}{x and y limits for the contour plot, use when both are same
#@param mainminmax  whether the min and max values should be shown in the title of plot}

\item{batchmax}{number of datapoints that can be computed at a time}

\item{out.col.name}{if a column needs to be selected from the function, specify it}

\item{out.name}{Selects with a $ the name from output to be used, for lists and data frames
#@param pretitle Text to be preappended to end of plot title
#@param posttitle Text to be appended to end of plot title
#@param title Title for the plot
#@param mainminmax_minmax Whether [min,max]= should be shown in title or just the numbers}

\item{pts}{Points to plot on top of contour}

\item{...}{Passed to cf_grid}
}
\description{
A contour plot of the given function without sidebar by default.
It calls the function `cf_grid` to make the actual plot.
}
\examples{
gcf_func(function(x){x[1]*x[2]})
gcf_func(function(x)(exp(-(x[1]-.5)^2-5*(x[2]-.5)^2)))
gcf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=TRUE, color.palette=terrain.colors)
gcf_func(function(xx){exp(-sum((xx-.5)^2/.1))}, bar=TRUE, mainminmax=TRUE)
gcf_func(function(x)(exp(-(x[1]-.5)^2-5*(x[2]-.5)^2)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cf_4dim.R
\name{cf_4dim}
\alias{cf_4dim}
\title{Plot 2D contour slices of four dimensional functions}
\usage{
cf_4dim(func, over = c(1, 2), nover = 5, nover1 = nover,
  nover2 = nover, low = rep(0, 4), high = rep(1, 4),
  same_scale = TRUE, n = 20, batchmax = 1,
  var_names = c(expression(), lapply(1:4, function(ti)
  bquote(x[.(ti)]))), pts = NULL, axes = TRUE, key.axes, key.title,
  nlevels = 20, color.palette = cm.colors.strong, edge_width = 0.04,
  cex.var_names = 1.3, bar = TRUE, bar_width = 0.2, over_srt = c(0,
  90), ...)
}
\arguments{
\item{func}{A four-dimensional function to plot contours of}

\item{over}{Indices of the dimensions used for the outer grid}

\item{nover}{Number of grid points for the outer grid dimensions}

\item{nover1}{Number of grid points for the first outer grid dimension}

\item{nover2}{Number of grid points for the second outer grid dimension}

\item{low}{Low input value for each dimension}

\item{high}{High input value for each dimension}

\item{same_scale}{Should all contour plots be on the same scale?
Takes longer since it has to precalculate range of outputs.}

\item{n}{Number of points in grid on each dimension}

\item{batchmax}{number of datapoints that can be computed at a time}

\item{var_names}{Variable names to add to plot}

\item{pts}{Matrix of points to show on plot}

\item{axes}{axes}

\item{key.axes}{key for bar plot}

\item{key.title}{statements which add titles for the plot key.}

\item{nlevels}{Number of levels in contour scale}

\item{color.palette}{Color palette used for contour plots}

\item{edge_width}{How wide should edges with variable names be?
As proportion of screen section to left of bar.
Either single value for both edges, or length two vector.}

\item{cex.var_names}{Size of var_names printed on edges.}

\item{bar}{Should a bar be added on right when all on same_scale?}

\item{bar_width}{How wide should bar section of plot be?}

\item{over_srt}{Degrees of rotation for the axis labels. Vector of length two.}

\item{...}{Arguments passed to cf_func, and then probably through to cf_grid}
}
\description{
Plots a grid of contour plots.
Each contour plot is a contour over two dimensions with the remaining
two dimensions set to a value.
See cf_highdim for functions with more than 4 dimensions.
}
\examples{
cf_4dim(
  function(x) {x[1] + x[2]^2 + sin(2*pi*x[3])}
)

cf_4dim(function(x) x[1]*x[3] + sin(x[2]*x[4]), color.palette=heat.colors,
        nover1=3, nover2=8, cex.var_names = .5)
        
cf_4dim(function(x) x[1]*x[3] + sin(x[2]*x[4]), color.palette=topo.colors,
        nover1=3, nover2=8, cex.var_names = 1, over_srt = c(90,0),
        edge_width=c(.1, .2), nlevels = 5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cf.R
\name{cf}
\alias{cf}
\title{Make contour plot from data or function}
\usage{
cf(..., gg = FALSE)
}
\arguments{
\item{...}{Arguments to be passed to cf_func or cf_data based on 
data type of first argument. If D is given as argument, then it
is passed to cf_highdim.}

\item{gg}{Should ggplot2 be used instead of base graphics?}
}
\value{
Whatever is returned from other function, probably nothing.
Will be a ggplot2 object if using gg=TRUE.
}
\description{
Simpler function for making contours with cf package.
Won't give argument completion, so all must be specified
}
\examples{
cf(function(x){x[1]^2 - x[2]})
x <- runif(20)
y <- runif(20)
z <- exp(-(x-.5)^2-5*(y-.5)^2)# + rnorm(20,0,.05)
cf(x,y,z)
cf(function(x){x[1]^2 - x[2]}, D=3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cf_multicolor_title.R
\name{multicolor.title}
\alias{multicolor.title}
\title{Makes plot title using specified colors for the text}
\usage{
multicolor.title(main, col.main, collapse = "",
  cex.main = par()$cex.main)
}
\arguments{
\item{main}{Text to put in main title of plot}

\item{col.main}{Colors to use for the text}

\item{collapse}{What to put between elements of main, defaults to "" but " " might be appropriate}

\item{cex.main}{The size of the main title. 1.2 is default.}
}
\description{
Makes plot title using specified colors for the text
}
\examples{
plot(1:4)
multicolor.title(c('Black, ','red, ','green'),c(1,2,3))
}
